<?php
namespace Adherence\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="adherence")
 */
class Adherence
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = false;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=64, name="hash")
	 */
	protected $hash;

	/**
	 * @ORM\Column(type="string", length=20, name="source")
	 */
	protected $source;

	/**
	 * @ORM\Column(type="string", length=20, name="type")
	 */
	protected $type;

	/**
	 * @ORM\Column(type="string", length=64, name="file")
	 */
	protected $file;

	/**
	 * @ORM\Column(type="string", length=20, name="dealer_code")
	 */
	protected $dealerCode;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=true, name="company_id")
	 **/
	protected $company;

	/**
	 * @ORM\Column(type="datetime", nullable=true, name="date_sold");
	 */
	protected $dateSold;

	/**
	 * @ORM\Column(type="string", length=20, nullable=true, name="stock_no")
	 */
	protected $stockNumber;

	/**
	 * @ORM\Column(type="string", length=15, nullable=true, name="registration_number")
	 */
	protected $registrationNumber;

	/**
	 * @ORM\Column(type="string", length=40, nullable=true, name="vin_number")
	 */
	protected $vinNumber;

	/**
	 * @ORM\Column(type="string", length=40, nullable=true, name="engine_number")
	 */
	protected $engineNumber;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vehicle_make")
	 */
	protected $make;

	/**
	 * @ORM\Column(type="string", length=150, name="sold_to")
	 */
	protected $soldTo;

	/**
	 * @ORM\Column(type="decimal", scale=4, precision=13, nullable=false, name="amount", options={"unsigned"=false});
	 */
	protected $amount = 0.0;

	/**
	 * @ORM\OneToMany(targetEntity="\Adherence\Entity\AdherenceStock", mappedBy="adherence", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $stock;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;



	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->stock = new \Doctrine\Common\Collections\ArrayCollection();
	}

	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}


	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	public function fromArray($data)
	{
		isset($data['hash'])
			&& $this->hash = $data['hash'];
		isset($data['source'])
			&& $this->source = $data['source'];
		isset($data['type'])
			&& $this->type = $data['type'];
		isset($data['file'])
			&& $this->file = $data['file'];
		isset($data['dealerCode'])
			&& $this->dealerCode = $data['dealerCode'];
		isset($data['company'])
			&& $this->company = $data['company'];
		isset($data['dateSold'])
			&& $this->dateSold = $data['dateSold'];
		isset($data['stockNumber'])
			&& $this->stockNumber = $data['stockNumber'];
		isset($data['registrationNumber'])
			&& $this->registrationNumber = $data['registrationNumber'];
		isset($data['vinNumber'])
			&& $this->vinNumber = $data['vinNumber'];
		isset($data['engineNumber'])
			&& $this->engineNumber = $data['engineNumber'];
		isset($data['make'])
			&& $this->make = $data['make'];
		isset($data['soldTo'])
			&& $this->soldTo = $data['soldTo'];
		isset($data['amount'])
			&& $this->amount = $data['amount'];
	}

	public function toArray()
	{
		return array(
				'id'   					=> $this->id,
				'hash' 					=> $this->hash,
				'source' 				=> $this->source,
				'type' 					=> $this->type,
				'file' 					=> $this->file,
				'dealerCode' 			=> $this->dealerCode,
				'company' 				=> !is_null($this->company)
					? $this->company->toArray()
					: null,
				'dateSold' 				=> !is_null($this->dateSold)
					? $this->dateSold->format(\Utility\Registry::getConfigParam('DateFormat'))
					: null,
				'stockNumber' 			=> $this->stockNumber,
				'registrationNumber' 	=> $this->registrationNumber,
				'vinNumber' 			=> $this->vinNumber,
				'engineNumber' 			=> $this->engineNumber,
				'make' 					=> $this->make,
				'soldTo' 				=> $this->soldTo,
				'amount' 				=> $this->amount
		);
	}
}
