<?php
namespace Auction\DataBin;



/**
 * Manage Bid data.
 * @author andre.fourie
 */
class Bid extends \Workspace\Service\DataBin
{

	/**
	 * @var array
	 */
	protected $meta = array(
			'Base'        => 'Bid',
			'DatasetName' => 'bids',
			'Entity'      => '\Auction\Entity\Bid',
			'JobField'    => 'auction',
			'References'  => array(
					'auction' => '\Auction\Entity\Auction',
					'company' => '\Company\Entity\Company',
					'profile' => '\User\Entity\Profile',
					'autoBid' => '\Auction\Entity\AutoBid'
			),
			'Dependants'  => array()
	);
	/**
	 * @var array
	*/
	protected $metaCreate = array(
			'Type'          => 'Create',
			'Contract'      => 'Recurring',
			'ConditionalContract' => array(
					'noBidOnExpiredAuction'
			),
			'ExecuteAfter'  => array(
					'updateStatus'
			),
			'RequiredInput' => array(
					'Bid' => array(
							'amount' => 'Decimal'
					)
			),
			'OptionalInput' => array()
	);
	/**
	 * @var array
	*/
	protected $metaGrid = array(
	    'Type'          => 'Grid',
		'Export'        => true,
		'Builder'       => '\Auction\Report\AuctionGrid',
		'Writer'        => '\Utility\Export\ExcelReport',
		'Output'        => 'Download',
	    'Contract'      => 'Recurring',
		'Options'       => array(
				'Boolean' => array(
						'ExportToExcel' => false
				)
		),
	    'RequiredInput' => array(),
	    'OptionalInput' => array(
	        'Grid'  => array(
	            'NumberOfRecords' => 'Integer',
	            'Page'            => 'Integer',
	            'Filter'          => 'Array',
	            'OrderBy'         => 'Array'
	        )
	    ),
	    'Base'            => 'bid',
	    'NumberOfRecords' => 1000,
        'Query'           => 'SELECT [SELECTION]
				              FROM \Auction\Entity\Bid bid
				              JOIN bid.company company
				              JOIN bid.profile profile
        					  LEFT JOIN bid.autoBid autoBid
				              [WHERE] [ORDER]',
	    'Selection'       => 'bid, autoBid, company, profile',
	    'Filter'          => array(),
	    'OrderBy'         => array('bid.amount' => 'DESC'),
	    'Fields'          => array(
				                'id',
				                'amount',
				                'created' => 'DateTime',
				                'status',
				                'company' => array(
				                    'name'
				                ),
				                'profile' => array(
				                	'firstName',
				                	'familyName',
				                	'email'
				                ),
				                'autoBid' => array(
				                    'amount' => 'DateTime'
				                )
				            )
	);

}