<?php
namespace Auction\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A reply concerning an item on question..
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="auction_reply")
 */
class Reply
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="Question")
	 * @ORM\JoinColumn(nullable=false, name="auction_question_id")
	 **/
	protected $question;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=false, name="profile_id")
	 **/
	protected $profile;


	/* ------------------------------------ Reply ------------------------------------ */
	/**
	 * @ORM\Column(type="text")
	 */
	protected $reply = '';


	/* ------------------------------------ Tracking ------------------------------------ */

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		is_null($this->profile)
			&& $this->profile = \Utility\Registry::resolveProfileContext($this->profile);
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['profile']))
			&& $data['profile'] = (in_array('profile', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->profile)
				? (!$showIdentifiers ? $this->profile->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->profile->id)
				: null;
		($includeAll || isset($intersect['reply']))
			&& $data['reply'] = $this->reply;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['question'])
			&& $this->question = $data['question'];
		isset($data['profile'])
			&& $this->profile = $data['profile'];
		isset($data['reply'])
			&& $this->reply = $data['reply'];
	}



}

