<?php
namespace Company\Entity;

use Doctrine\ORM\Mapping as ORM;


/**
 * A company.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="company")
 */
class Company
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;

	/**
	 * Valid dealer types values.
	 */
	const DEALERTYPE_DEALER       = 'Dealer';
	const DEALERTYPE_TRADER       = 'Trader';
	const DEALERTYPE_TRADE_CENTER = 'Trade Center';

	/**
	 * Valid client types values.
	 */
	const CLIENTTYPE_BUYER  = 'Buyer';
	const CLIENTTYPE_SELLER = 'Buyer & Seller';

	/**
	 * Valid company types values.
	 */
	const COMPANYTYPE_LTD       = 'Limited';
	const COMPANYTYPE_PTY_LTD   = 'PTY Limited';
	const COMPANYTYPE_CC        = 'Closed Corporation';
	const COMPANYTYPE_SOLE_PROP = 'Sole Proprietor';

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=100)
	 */
	protected $name;

	/**
	 * @ORM\Column(type="string", length=100, nullable=true, name="business_name")
	 */
	protected $businessName;

	/**
	 * @ORM\ManyToOne(targetEntity="\Location\Entity\Region")
	 * @ORM\JoinColumn( name="lib_region_id")
	 **/
	protected $region;

	/**
	 * @ORM\ManyToOne(targetEntity="\Location\Entity\Town")
	 * @ORM\JoinColumn(name="lib_town_id")
	 **/
	protected $city;

	/**
	 * @ORM\Column(type="string", length=10, name="postal_code")
	 */
	protected $postalCode;

	/**
	 * @ORM\Column(type="string", length=75)
	 */
	protected $street;

	/**
	 * @ORM\ManyToOne(targetEntity="\Location\Entity\Town")
	 * @ORM\JoinColumn(nullable=true, name="billing_lib_town_id")
	 **/
	protected $billingCity;

	/**
	 * @ORM\Column(type="string", length=10, nullable=true, name="billing_postal_code")
	 */
	protected $billingPostalCode;

	/**
	 * @ORM\Column(type="string", length=75, nullable=true, name="billing_street")
	 */
	protected $billingStreet;

	/**
	 * @ORM\ManyToOne(targetEntity="\Location\Entity\Town")
	 * @ORM\JoinColumn(nullable=true, name="auction_lib_town_id")
	 **/
	protected $auctionCity;

	/**
	 * @ORM\Column(type="string", length=10, nullable=true, name="auction_postal_code")
	 */
	protected $auctionPostalCode;

	/**
	 * @ORM\Column(type="string", length=75, nullable=true, name="auction_street")
	 */
	protected $auctionStreet;

	/**
	 * @ORM\Column(type="string", length=25, name="dealer_type")
	 */
	protected $dealerType = 'Dealer';

	/**
	 * @ORM\Column(type="string", length=25, name="client_type")
	 */
	protected $clientType = 'Buyer';

	/**
	 * @ORM\Column(type="string", length=100, nullable=true, name="company_registration_number")
	 */
	protected $companyRegistrationNumber;

	/**
	 * @ORM\Column(type="string", nullable=true, length=20, name="vat_number")
	 */
	protected $vatNumber;

	/**
	 * @ORM\Column(type="string", nullable=true, length=50, name="turmi_no")
	 */
	protected $turmiNumber;

	/**
	 * @ORM\Column(type="string", nullable=true, length=50, name="dealer_stock_number")
	 */
	protected $dealerStockNumber;

	/**
	 * @ORM\ManyToOne(targetEntity="Company")
	 * @ORM\JoinColumn(nullable=true, name="trade_center_dealer_id")
	 **/
	protected $tradeCenter;

	/**
	 * @ORM\Column(type="boolean", name="trade_exchange_link");
	 */
	protected $tradeExchangeLink = false;

	/**
	 * @ORM\ManyToOne(targetEntity="Group")
	 * @ORM\JoinColumn(name="company_group_id")
	 **/
	protected $group;

	/**
	 * @ORM\ManyToOne(targetEntity="GroupDivision")
	 * @ORM\JoinColumn(nullable=true, name="company_group_division_id")
	 **/
	protected $groupDivision;

	/**
	 * @ORM\ManyToOne(targetEntity="\Config\Entity\RegionalManager")
	 * @ORM\JoinColumn(nullable=true, name="regional_manager_id")
	 **/
	protected $regionalManager;

	/**
	 * @ORM\Column(type="string", nullable=true, length=25, name="company_type")
	 */
	protected $companyType;

	/**
	 * @ORM\Column(type="string", nullable=true, length=25, name="company_turnover")
	 */
	protected $turnover;

	/**
	 * @ORM\OneToOne(targetEntity="\Person\Entity\Contact")
	 * @ORM\JoinColumn(nullable=true, name="lib_contact_id")
	 **/
	protected $contact;

	/**
	 * @ORM\OneToOne(targetEntity="\Person\Entity\Contact")
	 * @ORM\JoinColumn(nullable=true, name="billing_lib_contact_id")
	 **/
	protected $billingContact;

	/**
	 * @ORM\ManyToOne(targetEntity="FoundMethod")
	 * @ORM\JoinColumn(nullable=true, name="company_found_method_id")
	 **/
	protected $foundMethod;

	/**
	 * @ORM\ManyToOne(targetEntity="FoundMethodDetail")
	 * @ORM\JoinColumn(nullable=true, name="company_found_method_detail_id")
	 **/
	protected $foundMethodDetail;

	/**
	 * @ORM\OneToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="lib_photo_id")
	 **/
	protected $dealershipPhoto;

	/**
	 * @ORM\Column(type="boolean", name="price_guide");
	 */
	protected $priceGuide = false;

	/**
	 * @ORM\Column(type="boolean", name="club_invite");
	 */
	protected $clubInvite = false;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Document")
	 * @ORM\JoinColumn(nullable=true, name="stock_cert_lib_document_id")
	 **/
	private $docStockCertificate;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Document")
	 * @ORM\JoinColumn(nullable=true, name="address_proof_lib_document_id")
	 **/
	private $docAddressProof;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Document")
	 * @ORM\JoinColumn(nullable=true, name="copy_of_id_lib_document_id")
	 **/
	private $docCopyOfId;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Document")
	 * @ORM\JoinColumn(nullable=true, name="copy_of_director_id_lib_document_id")
	 **/
	private $docCopyOfDirectorId;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Document")
	 * @ORM\JoinColumn(nullable=true, name="company_reg_lib_document_id")
	 **/
	private $docCompanyRegistration;

	/**
	 * @ORM\Column(nullable=true, type="string", length=64, name="automate_file")
	 */
	protected $automateFile;

	/**
	 * @ORM\Column(nullable=true, type="string", length=20, name="automate_dealer_code")
	 */
	protected $automateDealerCode;

	/**
	 * @ORM\Column(nullable=true, type="string", length=64, name="autoline_file")
	 */
	protected $autolineFile;

	/**
	 * @ORM\Column(nullable=true, type="string", length=20, name="autoline_dealer_code")
	 */
	protected $autolineDealerCode;

	/**
	 * @ORM\Column(nullable=true, type="string", length=64, name="amhgroup_file")
	 */
	protected $amhgroupFile;

	/**
	 * @ORM\Column(nullable=true, type="string", length=20, name="amhgroup_dealer_code")
	 */
	protected $amhgroupDealerCode;


	/**
	 * @ORM\OneToMany(targetEntity="\User\Entity\Profile", mappedBy="company", fetch="EXTRA_LAZY")
	 **/
	private $profiles;

	/**
	 * @ORM\OneToMany(targetEntity="CompanyRetail", mappedBy="company", fetch="EXTRA_LAZY")
	 **/
	private $retailers;

	/**
	 * @ORM\OneToMany(targetEntity="CompanyOwner", mappedBy="company", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	private $owners;

	/**
	 * @ORM\Column(type="string", length=10, name="status")
	 */
	protected $jobState = 'New';

	/**
	 * @ORM\Column(type="string", nullable=true, length=250, name="status_reason")
	 */
	protected $statusReason;

	/**
	 * @ORM\Column(type="boolean",  name="public_centre");
	 */
	protected $publicCentre = false;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->profiles  = new \Doctrine\Common\Collections\ArrayCollection();
		$this->retailers = new \Doctrine\Common\Collections\ArrayCollection();
		$this->owners    = new \Doctrine\Common\Collections\ArrayCollection();
	}


	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed  $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array   $expand
	 * @param array   $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
		array $expand = array(), array $intersect = array(),
		$showIdentifiers = false, $expandAll = 0
	)
	{
		$intersect      = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll     = empty($intersect);
		$recExpand      = $expand;
		foreach ($recExpand as $i => $val)
		{
			if ('company' == $val || 'tradeCenter' == $val)
			{
				unset($recExpand[$i]);
			}
		}
		$data = array();
		($includeAll || isset($intersect['id']))
		&& $data['id'] = $this->id;
		($includeAll || isset($intersect['name']))
		&& $data['name'] = $this->name;
		($includeAll || isset($intersect['businessName']))
		&& $data['businessName'] = $this->businessName;

		($includeAll || isset($intersect['region']))
		&& $data['region'] = (in_array('region', $expand) || $expandAll || $showIdentifiers)
		                     && !is_null($this->region)
			? (!$showIdentifiers || in_array('region', $expand) ? $this->region->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->region->id)
			: null;
		($includeAll || isset($intersect['city']))
		&& $data['city'] = (in_array('city', $expand) || $expandAll || $showIdentifiers)
		                   && !is_null($this->city)
			? (!$showIdentifiers || in_array('city', $expand) ? $this->city->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->city->id)
			: null;


		($includeAll || isset($intersect['postalCode']))
		&& $data['postalCode'] = $this->postalCode;
		($includeAll || isset($intersect['street']))
		&& $data['street'] = $this->street;

		($includeAll || isset($intersect['billingCity']))
		&& $data['billingCity'] = (in_array('billingCity', $expand) || $expandAll || $showIdentifiers)
		                          && !is_null($this->billingCity)
			? (!$showIdentifiers || in_array('billingCity', $expand) ? $this->billingCity->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->billingCity->id)
			: null;
		($includeAll || isset($intersect['billingPostalCode']))
		&& $data['billingPostalCode'] = $this->billingPostalCode;
		($includeAll || isset($intersect['billingStreet']))
		&& $data['billingStreet'] = $this->billingStreet;
		($includeAll || isset($intersect['auctionCity']))
		&& $data['auctionCity'] = (in_array('auctionCity', $expand) || $expandAll || $showIdentifiers)
		                          && !is_null($this->auctionCity)
			? (!$showIdentifiers || in_array('auctionCity', $expand) ? $this->auctionCity->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->auctionCity->id)
			: null;
		($includeAll || isset($intersect['auctionPostalCode']))
		&& $data['auctionPostalCode'] = $this->auctionPostalCode;
		($includeAll || isset($intersect['auctionStreet']))
		&& $data['auctionStreet'] = $this->auctionStreet;

		($includeAll || isset($intersect['dealerType']))
		&& $data['dealerType'] = $this->dealerType;
		($includeAll || isset($intersect['clientType']))
		&& $data['clientType'] = $this->clientType;
		($includeAll || isset($intersect['companyRegistrationNumber']))
		&& $data['companyRegistrationNumber'] = $this->companyRegistrationNumber;
		($includeAll || isset($intersect['vatNumber']))
		&& $data['vatNumber'] = $this->vatNumber;
		($includeAll || isset($intersect['turmiNumber']))
		&& $data['turmiNumber'] = $this->turmiNumber;
		($includeAll || isset($intersect['dealerStockNumber']))
		&& $data['dealerStockNumber'] = $this->dealerStockNumber;
		($includeAll || isset($intersect['tradeCenter']))
		&& $data['tradeCenter'] = (in_array('tradeCenter', $expand) || $expandAll || $showIdentifiers)
		                          && !is_null($this->tradeCenter)
			? (!$showIdentifiers || in_array('tradeCenter', $expand)
				? $this->tradeCenter->toArray(
					array(), array('id', 'name'), $showIdentifiers, 0
				)
				: $this->tradeCenter->id)
			: null;
		($includeAll || isset($intersect['tradeExchangeLink']))
		&& $data['tradeExchangeLink'] = $this->tradeExchangeLink;
		($includeAll || isset($intersect['group']))
		&& $data['group'] = (in_array('group', $expand) || $expandAll || $showIdentifiers)
		                    && !is_null($this->group)
			? (!$showIdentifiers || in_array('group', $expand) ? $this->group->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->group->id)
			: null;
		($includeAll || isset($intersect['groupDivision']))
		&& $data['groupDivision'] = (in_array('groupDivision', $expand) || $expandAll || $showIdentifiers)
		                            && !is_null($this->groupDivision)
			? (!$showIdentifiers || in_array('groupDivision', $expand) ? $this->groupDivision->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->groupDivision->id)
			: null;
		($includeAll || isset($intersect['regionalManager']))
		&& $data['regionalManager'] = (in_array('regionalManager', $expand) || $expandAll || $showIdentifiers)
		                              && !is_null($this->regionalManager)
			? (!$showIdentifiers || in_array('regionalManager', $expand) ? $this->regionalManager->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->regionalManager->id)
			: null;
		($includeAll || isset($intersect['companyType']))
		&& $data['companyType'] = $this->companyType;
		($includeAll || isset($intersect['turnover']))
		&& $data['turnover'] = $this->turnover;
		($includeAll || isset($intersect['contact']))
		&& $data['contact'] = (in_array('contact', $expand) || $expandAll || $showIdentifiers)
		                      && !is_null($this->contact)
			? (!$showIdentifiers || in_array('contact', $expand) ? $this->contact->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->contact->id)
			: null;
		($includeAll || isset($intersect['billingContact']))
		&& $data['billingContact'] = (in_array('billingContact', $expand) || $expandAll || $showIdentifiers)
		                             && !is_null($this->billingContact)
			? (!$showIdentifiers || in_array('billingContact', $expand) ? $this->billingContact->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->billingContact->id)
			: null;
		($includeAll || isset($intersect['foundMethod']))
		&& $data['foundMethod'] = (in_array('foundMethod', $expand) || $expandAll || $showIdentifiers)
		                          && !is_null($this->foundMethod)
			? (!$showIdentifiers || in_array('foundMethod', $expand) ? $this->foundMethod->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->foundMethod->id)
			: null;
		($includeAll || isset($intersect['foundMethodDetail']))
		&& $data['foundMethodDetail'] = (in_array('foundMethodDetail', $expand) || $expandAll || $showIdentifiers)
		                                && !is_null($this->foundMethodDetail)
			? (!$showIdentifiers || in_array('foundMethodDetail', $expand) ? $this->foundMethodDetail->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->foundMethodDetail->id)
			: null;
		($includeAll || isset($intersect['dealershipPhoto']))
		&& $data['dealershipPhoto'] = (in_array('dealershipPhoto', $expand) || $expandAll || $showIdentifiers)
		                              && !is_null($this->dealershipPhoto)
			? (!$showIdentifiers || in_array('dealershipPhoto', $expand) ? $this->dealershipPhoto->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->dealershipPhoto->id)
			: null;
		($includeAll || isset($intersect['priceGuide']))
		&& $data['priceGuide'] = $this->priceGuide;


		($includeAll || isset($intersect['publicCentre']))
		&& $data['publicCentre'] = $this->publicCentre;


		($includeAll || isset($intersect['clubInvite']))
		&& $data['clubInvite'] = $this->clubInvite;

		($includeAll || isset($intersect['docStockCertificate']))
		&& $data['docStockCertificate'] = (in_array('docStockCertificate', $expand) || $expandAll || $showIdentifiers)
		                                  && !is_null($this->docStockCertificate)
			? (!$showIdentifiers || in_array('docStockCertificate', $expand) ? $this->docStockCertificate->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->docStockCertificate->id)
			: null;
		($includeAll || isset($intersect['docAddressProof']))
		&& $data['docAddressProof'] = (in_array('docAddressProof', $expand) || $expandAll || $showIdentifiers)
		                              && !is_null($this->docAddressProof)
			? (!$showIdentifiers || in_array('docAddressProof', $expand) ? $this->docAddressProof->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->docAddressProof->id)
			: null;
		($includeAll || isset($intersect['docCopyOfId']))
		&& $data['docCopyOfId'] = (in_array('docCopyOfId', $expand) || $expandAll || $showIdentifiers)
		                          && !is_null($this->docCopyOfId)
			? (!$showIdentifiers || in_array('docCopyOfId', $expand) ? $this->docCopyOfId->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->docCopyOfId->id)
			: null;
		($includeAll || isset($intersect['docCopyOfDirectorId']))
		&& $data['docCopyOfDirectorId'] = (in_array('docCopyOfDirectorId', $expand) || $expandAll || $showIdentifiers)
		                                  && !is_null($this->docCopyOfDirectorId)
			? (!$showIdentifiers || in_array('docCopyOfDirectorId', $expand) ? $this->docCopyOfDirectorId->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->docCopyOfDirectorId->id)
			: null;
		($includeAll || isset($intersect['docCompanyRegistration']))
		&& $data['docCompanyRegistration'] = (in_array('docCompanyRegistration', $expand) || $expandAll || $showIdentifiers)
		                                     && !is_null($this->docCompanyRegistration)
			? (!$showIdentifiers || in_array('docCompanyRegistration', $expand) ? $this->docCompanyRegistration->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->docCompanyRegistration->id)
			: null;

		($includeAll || isset($intersect['automateFile']))
		&& $data['automateFile'] = $this->automateFile;
		($includeAll || isset($intersect['automateDealerCode']))
		&& $data['automateDealerCode'] = $this->automateDealerCode;

		($includeAll || isset($intersect['autolineFile']))
		&& $data['autolineFile'] = $this->autolineFile;
		($includeAll || isset($intersect['autolineDealerCode']))
		&& $data['autolineDealerCode'] = $this->autolineDealerCode;

		($includeAll || isset($intersect['amhgroupFile']))
		&& $data['amhgroupFile'] = $this->amhgroupFile;
		($includeAll || isset($intersect['amhgroupDealerCode']))
		&& $data['amhgroupDealerCode'] = $this->amhgroupDealerCode;

		($includeAll || isset($intersect['jobState']))
		&& $data['jobState'] = $this->jobState;
		($includeAll || isset($intersect['statusReason']))
		&& $data['statusReason'] = $this->statusReason;
		($includeAll || isset($intersect['created']))
		&& $data['created'] = !is_null($this->created)
			? $this->created->format($dateTimeFormat)
			: null;
		($includeAll || isset($intersect['updated']))
		&& $data['updated'] = !is_null($this->updated)
			? $this->updated->format($dateTimeFormat)
			: null;
		return $data;
	}

	/**
	 * Internal utility to change owners collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	protected function ownersToArray($showIdentifiers = false)
	{
		$data     = array();
		$iterator = $this->owners->getIterator();
		foreach ($iterator as $owner)
		{
			$data[] = $showIdentifiers
				? array('id' => $owner->id)
				: $owner->toArray();
		}
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
		&& $this->id = $data['id'];
		if (isset($data['name']))
		{
			$this->name = $data['name'];

			#-> Update de-normalized auction data.
			$em = \Utility\Registry::getEntityManager();
			if (!is_null($em))
			{
				$em->createQuery(
					'UPDATE \Auction\Entity\Auction a '
					. "SET a.auctionCompanyName = :name "
					. 'WHERE a.jobState = :jobState '
					. ' AND a.auctionCompanyId = :id'
				)
					->setParameter('jobState', 'Active')
					->setParameter('id', $this->id)
					->setParameter('name', $this->name)
					->execute();
			}
		}
		isset($data['businessName'])
		&& $this->businessName = $data['businessName'];
		isset($data['region'])
		&& $this->region = $data['region'];
		isset($data['city'])
		&& $this->city = $data['city'];


		isset($data['publicCentre'])
		&& $this->publicCentre = $data['publicCentre'];

		isset($data['postalCode'])
		&& $this->postalCode = $data['postalCode'];
		isset($data['street'])
		&& $this->street = $data['street'];
		isset($data['billingCity'])
		&& $this->billingCity = $data['billingCity'];
		isset($data['billingPostalCode'])
		&& $this->billingPostalCode = $data['billingPostalCode'];
		isset($data['billingStreet'])
		&& $this->billingStreet = $data['billingStreet'];
		isset($data['auctionCity'])
		&& $this->auctionCity = $data['auctionCity'];
		isset($data['auctionPostalCode'])
		&& $this->auctionPostalCode = $data['auctionPostalCode'];
		isset($data['auctionStreet'])
		&& $this->auctionStreet = $data['auctionStreet'];
		isset($data['dealerType'])
		&& $this->dealerType = $data['dealerType'];
		isset($data['clientType'])
		&& $this->clientType = $data['clientType'];
		isset($data['companyRegistrationNumber'])
		&& $this->companyRegistrationNumber = $data['companyRegistrationNumber'];
		isset($data['vatNumber'])
		&& $this->vatNumber = $data['vatNumber'];
		isset($data['turmiNumber'])
		&& $this->turmiNumber = $data['turmiNumber'];
		isset($data['dealerStockNumber'])
		&& $this->dealerStockNumber = $data['dealerStockNumber'];
		key_exists('tradeCenter', $data)
		&& $this->tradeCenter = $data['tradeCenter'];
		isset($data['tradeExchangeLink'])
		&& $this->tradeExchangeLink = $data['tradeExchangeLink'];
		isset($data['group'])
		&& $this->group = $data['group'];
		isset($data['groupDivision'])
		&& $this->groupDivision = $data['groupDivision'];
		isset($data['regionalManager'])
		&& $this->regionalManager = $data['regionalManager'];
		isset($data['companyType'])
		&& $this->companyType = $data['companyType'];
		isset($data['turnover'])
		&& $this->turnover = $data['turnover'];
		isset($data['contact'])
		&& $this->contact = $data['contact'];
		isset($data['foundMethod'])
		&& $this->foundMethod = $data['foundMethod'];
		isset($data['foundMethodDetail'])
		&& $this->foundMethodDetail = $data['foundMethodDetail'];
		isset($data['dealershipPhoto'])
		&& $this->dealershipPhoto = $data['dealershipPhoto'];
		isset($data['priceGuide'])
		&& $this->priceGuide = $data['priceGuide'];


		isset($data['publicCentre'])
		&& $this->publicCentre = $data['publicCentre'];


		isset($data['clubInvite'])
		&& $this->clubInvite = $data['clubInvite'];
		isset($data['statusReason'])
		&& $this->statusReason = $data['statusReason'];

		isset($data['docCopyOfId'])
		&& $this->docCopyOfId = $data['docCopyOfId'];
		isset($data['docCopyOfDirectorId'])
		&& $this->docCopyOfDirectorId = $data['docCopyOfDirectorId'];
		isset($data['docCompanyRegistration'])
		&& $this->docCompanyRegistration = $data['docCompanyRegistration'];
		isset($data['docStockCertificate'])
		&& $this->docStockCertificate = $data['docStockCertificate'];
		isset($data['docAddressProof'])
		&& $this->docAddressProof = $data['docAddressProof'];

		isset($data['automateFile'])
		&& $this->automateFile = $data['automateFile'];
		isset($data['automateDealerCode'])
		&& $this->automateDealerCode = $data['automateDealerCode'];

		isset($data['autolineFile'])
		&& $this->autolineFile = $data['autolineFile'];
		isset($data['autolineDealerCode'])
		&& $this->autolineDealerCode = $data['autolineDealerCode'];

		isset($data['amhgroupFile'])
		&& $this->amhgroupFile = $data['amhgroupFile'];
		isset($data['amhgroupDealerCode'])
		&& $this->amhgroupDealerCode = $data['amhgroupDealerCode'];


		if (isset($data['owners']))
		{
			$em       = \Utility\Registry::getEntityManager();
			$iterator = $this->owners->getIterator();
			foreach ($iterator as $ownr)
			{
				$em->remove($ownr);
			}
			foreach ($data['owners'] as $ownr)
			{
				$companyOwnr             = new \Company\Entity\CompanyOwner();
				$companyOwnr->firstName  = $ownr['firstName'];
				$companyOwnr->familyName = $ownr['familyName'];
				$companyOwnr->idNumber   = $ownr['idNumber'];
				$companyOwnr->company    = $this;
				$this->owners->add($companyOwnr);
			}
		}
	}

}
