<?php
namespace Company\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="company_owner")
 */
class CompanyOwner
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = false;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="Company", inversedBy="owners")
	 * @ORM\JoinColumn(name="company_id", referencedColumnName="id")
	 **/
	protected $company;

	/**
	 * @ORM\Column(type="string", length=100, name="first_name")
	 */
	protected $firstName;

	/**
	 * @ORM\Column(type="string", length=100, name="family_name")
	 */
	protected $familyName;

	/**
	 * @ORM\Column(type="string", length=13, name="id_number")
	 */
	protected $idNumber;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;



	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}


	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	public function fromArray($data)
	{
		isset($data['company'])
			&& $this->company = $data['company'];
		isset($data['firstName'])
			&& $this->firstName = $data['firstName'];
		isset($data['familyName'])
			&& $this->familyName = $data['familyName'];
		isset($data['idNumber'])
			&& $this->idNumber = $data['idNumber'];
	}

	public function toArray()
	{
		$dateTimeFormat = \Utility\Definitions\Locale::getDateTimeFormat();
		$data = array();

		$data['company']	= $this->company;
		$data['firstName']	= $this->firstName;
		$data['familyName']	= $this->familyName;
		$data['idNumber']	= $this->idNumber;
		$data['created']	= $this->created->format($dateTimeFormat);
		$data['updated']	= !is_null($this->updated)
			? $this->updated->format($dateTimeFormat)
			: '';

		return $data;
	}
}
