<?php
namespace Company\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="company_retail")
 */
class CompanyRetail
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="Company", inversedBy="retailers")
	 * @ORM\JoinColumn(name="company_id", referencedColumnName="id")
	 **/
	protected $company;

	/**
	 * @ORM\ManyToOne(targetEntity="\Retail\Entity\Retail")
	 * @ORM\JoinColumn(name="retail_id", referencedColumnName="id")
	 **/
	protected $retail;

	/**
	 * @ORM\Column(type="string", length=150)
	 */
	protected $name;

	/**
	 * @ORM\Column(type="string", length=150)
	 */
	protected $username;

	/**
	 * @ORM\Column(type="string", length=64)
	 */
	protected $password;


	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = 0;



	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}


	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
		$this->company = \Utility\Registry::resolveCompanyContext($this->company);
		if (empty($this->name) || is_null($this->name))
		{
			$this->name = $this->retail->name;
		}
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	public function fromArray($data)
	{
		isset($data['company'])
			&& $this->company = $data['company'];
		isset($data['retail'])
			&& $this->retail = $data['retail'];
		isset($data['name'])
			&& $this->name = $data['name'];
		isset($data['username'])
			&& $this->username = $data['username'];
		isset($data['password'])
			&& $this->password = $data['password'];
		isset($data['archived'])
			&& $this->archived = $data['archived'];
	}

	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$includeAll = empty($intersect);
		$dateTimeFormat = \Utility\Definitions\Locale::getDateTimeFormat();
		$data = array();
		$data['id']			= $this->id;
		($includeAll || isset($intersect['company']))
			&& $data['company'] = (in_array('company', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->company)
				? (!$showIdentifiers || in_array('company', $expand) ? $this->company->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->company->id)
				: null;
		($includeAll || isset($intersect['retail']))
			&& $data['retail'] = (in_array('retail', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->retail)
				? (!$showIdentifiers || in_array('retail', $expand) ? $this->retail->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->retail->id)
				: null;
		$data['name']	    = $this->name;
		$data['username']	= $this->username;
		$data['password']	= $this->password;
		$data['created']	= $this->created->format($dateTimeFormat);
		$data['updated']	= !is_null($this->updated)
			? $this->updated->format($dateTimeFormat)
			: null;
		$data['archived']	= $this->archived;

		return $data;
	}
}
