<?php
namespace Company\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A company mobile device registration.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="company_device_registration")
 */
class DeviceRegistration
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=false, name="company_id")
	 **/
	protected $company;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=false, name="profile_id")
	 **/
	protected $profile;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Device")
	 * @ORM\JoinColumn(nullable=false, name="device_id")
	 **/
	protected $device;

	/**
	 * @ORM\Column(type="string", length=100, name="push_key", nullable=false)
	 */
	protected $pushKey;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;





	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$recExpand = $expand;
		foreach($recExpand as $i => $val)
		{
			if ('company' == $val || 'tradeCenter' == $val)
			{
				unset($recExpand[$i]);
			}
		}
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['pushKey']))
			&& $data['pushKey'] = $this->pushKey;
		($includeAll || isset($intersect['company']))
			&& $data['company'] = (in_array('company', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->company)
				? (!$showIdentifiers || in_array('company', $expand) ? $this->company->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->company->id)
				: null;
		($includeAll || isset($intersect['profile']))
			&& $data['profile'] = (in_array('profile', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->profile)
				? (!$showIdentifiers || in_array('profile', $expand) ? $this->profile->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->profile->id)
				: null;
		($includeAll || isset($intersect['device']))
			&& $data['device'] = (in_array('device', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->device)
				? (!$showIdentifiers || in_array('device', $expand) ? $this->device->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->device->id)
				: null;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['pushKey'])
			&& $this->pushKey = $data['pushKey'];
		isset($data['company'])
			&& $this->company = $data['company'];
		isset($data['profile'])
			&& $this->profile = $data['profile'];
		isset($data['device'])
			&& $this->device = $data['device'];
	}

}
