<?php
namespace PriceGuide\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A price guide club.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="price_guide_club")
 */
class Club
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=false, name="company_id")
	 **/
	protected $company;


 	/* ------------------------------------ Customer ------------------------------------ */
	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="name")
	 */
	protected $name;


	/* ------------------------------------ Linkage ------------------------------------ */
	/**
	 * @ORM\OneToMany(targetEntity="ClubStock", mappedBy="club", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $stockItems;
	/**
	 * @ORM\OneToMany(targetEntity="Member", mappedBy="club", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $members;


	/* ------------------------------------ Tracking ------------------------------------ */

	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="status");
	 */
	protected $status = 'Active';

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean", name="use_as_default");
	 */
	protected $useAsDefault = false;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		if (is_null($this->company))
		{
			$this->company = \Utility\Registry::resolveCompanyContext($this->company);
		}
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['company']))
			&& $data['company'] = (in_array('company', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->company)
				? (!$showIdentifiers ? $this->company->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->company->id)
				: null;
		($includeAll || isset($intersect['name']))
			&& $data['name'] = $this->name;
		($includeAll || isset($intersect['status']))
			&& $data['status'] = $this->status;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['useAsDefault']))
			&& $data['useAsDefault'] = $this->useAsDefault;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['company'])
			&& $this->company = $data['company'];
		isset($data['name'])
			&& $this->name = $data['name'];
		isset($data['status'])
			&& $this->status = $data['status'];
		isset($data['useAsDefault'])
			&& $this->useAsDefault = $data['useAsDefault'];
	}



}

