<?php
namespace PriceGuide\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * @ORM\Entity
 * @ORM\Table(name="price_guide_club_members")
 */
class ClubMember
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="Club", inversedBy="members")
	 * @ORM\JoinColumn(name="price_guide_club_id", referencedColumnName="id")
	 **/
	protected $club;

	/**
	 * @ORM\ManyToOne(targetEntity="Member", inversedBy="clubs")
	 * @ORM\JoinColumn(name="price_guide_member_id", referencedColumnName="id")
	 **/
	protected $member;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(array $expand = array(), array $intersect = array())
	{
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['priceGuideStock']))
			&& $data['priceGuideStock'] = $this->priceGuideStock->toArray($expand, $intersect);
		($includeAll || isset($intersect['club']))
			&& $data['club'] = $this->club->toArray($expand, $intersect);
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['priceGuideStock'])
			&& $this->priceGuideStock  = $data['priceGuideStock'];
		isset($data['club'])
			&& $this->club  = $data['club'];
	}

}
