<?php
namespace PriceGuide;



/**
 * A workflow.
 * @author andre.fourie
 */
class Workflow extends \Workspace\Workflow
{

	/**
	 * @var string
	 */
	protected $namespace = __NAMESPACE__;
	/**
	 * @var string
	 */
	protected $parentFeature = 'Stock';
	/**
	 * @var string
	 */
	protected $rootEntity    = 'Stock';
	/**
	 * @var string
	 */
	protected $rootIdField   = 'stock';
	/**
	 * @var string
	 */
	protected $initState     = 'Open4Offers';
	/**
	 * @var string
	 */
	protected $initTrigger   = false;
	/**
	 * @var string
	 */
	protected $primaryEntity = 'PriceGuide';
	/**
	 * @var array
	 */
	protected $entityMap = array(
			'PriceGuide' => '\PriceGuide\Entity\PriceGuide',
			'Stock'     => '\Stock\Entity\Stock'
	);
	/**
	 * @var array
	 */
	protected $linkMap = array();
	/**
	 * @var array
	 */
	protected $listen = array();
	/**
	 * @var array
	 */
	protected $reclaim = array();
	/**
	 * @var array
	 */
	protected $stateLabel = array(
			'UpdateableOffers' => 'Updateable Offers',
			'ValidOffers'      => 'Valid Offers',
			'OfferAccepted'     => 'Offer Accepted'
	);
	/**
	 * @var array
	 */
	protected $stateMap = array(
			'Global' => array(
				'Actions' => array(
					'PriceGuide.Grid'           	=> array(),
					'PriceGuide.AddToTradeClub' 	=> array(),
					'PriceGuide.SetupImperialClubs' => array(),
					'AllowedMember.SelectList'  	=> array(),
					'Club.SelectList'           	=> array(),
					'Club.Grid'                 	=> array(),
					'Club.Create'               	=> array(),
					'Club.Update'               	=> array(),
					'Club.Archive'              	=> array(),
					'Offer.Create'              	=> array(),
					'Offer.Update'              	=> array(),
					'Offer.Archive'             	=> array(),
					'Member.Grid'               	=> array(),
					'Member.MembershipGrid'     	=> array(),
					'Member.Invite'             	=> array(),
					'Member.Update'             	=> array(),
					'Member.UpdateStatus'       	=> array(),
					'Member.Archive'            	=> array(),
					'Member.Accept'             	=> array(),
					'Member.Decline'            	=> array(),
					'Member.Activate'           	=> array(),
					'Member.Deactivate'         	=> array(),
					'Invite.Grid'               	=> array(),
					'Invite.Archive'            	=> array(),
					'Invite.View'               	=> array(),
					'Invite.Update'             	=> array(),
					'PricingHistory.Grid'   		=> array()
				)
			),
			'Open4Offers' => array(
				'Actions' => array(
					'PriceGuide.View'                     => array(),
					'PriceGuide.Remove' 			      => array(),
					'PriceGuide.Expire' 			      => array(),
					'Pdf.Print'                           => array(),
					'PriceGuide.RequestBmwAccidentReport' => array(),
					'PriceGuide.RequestAccidentReport'    => array(),
					'Offer.List'                          => array(),
					'Offer.Create'                        => array(),
					'Offer.Update'                        => array(),
					'Offer.Archive'                       => array(),
					'Offer.Unarchive'                     => array(),
					'PricingHistory.Grid'   		=> array()
				),
				'Routes' => array(
					'Workflow.ReturnToStock'     => array(
							'Destination' => 'Stock.Handover'
					),
					'Workflow.ReturnToValuation' => array(
							'Destination' => 'Valuation.Handover'
					)
				)
			),
			'Updateable Offers' => array(
				'Actions' => array(
					'PriceGuide.View'                     => array(),
					'PriceGuide.Remove' 			      => array(),
					'PriceGuide.Expire' 			      => array(),
					'Pdf.Print'                           => array(),
					'Pdf.Print004'                        => array(),
					'PriceGuide.RequestBmwAccidentReport' => array(),
					'PriceGuide.RequestAccidentReport'    => array(),
					'Offer.List'                          => array(),
					'Offer.Create'                        => array(),
					'Offer.Update'                        => array(),
					'Offer.Archive'                       => array(),
					'Offer.Unarchive'                     => array(),
					'PricingHistory.Grid'   		=> array()
				),
				'Routes' => array(
					'Workflow.ReturnToStock'     => array(
							'Destination' => 'Stock.Handover'
					),
					'Workflow.ReturnToValuation' => array(
							'Destination' => 'Valuation.Handover'
					)
				)
			),
			'Valid Offers' => array(
				'Actions' => array(
					'PriceGuide.View'                     => array(),
					'Pdf.Print'                           => array(),
					'Pdf.Print004'                        => array(),
					'PriceGuide.RequestBmwAccidentReport' => array(),
					'PriceGuide.RequestAccidentReport'    => array(),
					'Offer.List'                          => array(),
					'Offer.View'                          => array(),
					'Offer.Archive'                       => array(),
					'Offer.Unarchive'                     => array(),
					'PricingHistory.Grid'   		=> array()
				),
				'Routes' => array(
					'PriceGuide.OfferAccepted' => array(
							'Destination' => 'This.OfferAccepted'
					),
					'Workflow.MoveToArchive'    => array(
							'Destination' => 'This.Archived'
					),
					'Workflow.ReturnToStock'     => array(
							'Destination' => 'Stock.Handover'
					),
					'Workflow.ReturnToValuation' => array(
							'Destination' => 'Valuation.Handover'
					)
				)
			),
			'Offer Accepted' => array(
				'Actions' => array(
					'PriceGuide.View'                     => array(),
					'Pdf.Print'                           => array(),
					'Pdf.Print004'                        => array(),
					'PriceGuide.RequestBmwAccidentReport' => array(),
					'PriceGuide.RequestAccidentReport'    => array(),
					'Offer.List'                          => array(),
					'Offer.View'                          => array(),
					'Offer.Archive'                       => array(),
					'Offer.Unarchive'                     => array(),
					'PricingHistory.Grid'   		=> array()
				),
				'Routes' => array(
					'Workflow.MoveToArchive'    => array(
							'Destination' => 'This.Archived'
					)
				)
			),
			'Archived' => array(
				'Actions' => array(
					'PriceGuide.View'  => array(),
					'Pdf.Print'        => array(),
					'Pdf.Print004'     => array(),
					'Offer.List'       => array(),
					'Offer.View'       => array(),
					'Offer.Archive'    => array(),
					'Offer.Unarchive'  => array(),
					'PricingHistory.Grid'   		=> array()
				),
				'Routes' => array(
					'Workflow.ReturnToStock'     => array(
							'Destination' => 'Stock.Handover'
					),
					'Workflow.ReturnToValuation' => array(
							'Destination' => 'Valuation.Handover'
					)
				)
			)
	);


}
