<?php

namespace Report\Report;

class LoadVsSoldExcel extends \Utility\Export\ExcelReport
{

	/**
	 * Constructor.
	 *
	 * @param \Utility\Service\Report $report
	 */
	public function __construct(\Utility\Service\Report $report) {
		$this->_fileName = str_replace( ' ', '-', $report->getTitle() );
		$objPHPExcel = new \PHPExcel();
		$objWriter = \PHPExcel_IOFactory::createWriter( $objPHPExcel, 'Excel2007' );
		$this->_objReport = $report;
		$this->_format = $report->getFormat();
		$this->_queries = $report->getQueries();
		$this->_headers = $report->getHeaders();
		$this->_fields = $report->getFields();
		$this->_totals = $report->getTotalFields();
		$this->_currencyFields = $report->getCurrencyFields();
		$this->_notes = $report->getNotes();
		$this->_chart = $report->getChart();
		$this->_data = $report->getData();
		$this->_objWriter = $objWriter;
		$this->_objPHPExcel = $objPHPExcel;
		$this->_objPHPExcel->getProperties()->setCreator( APPLICATION );
		$this->_objPHPExcel->getProperties()->setLastModifiedBy( APPLICATION );
		$this->_objPHPExcel->getProperties()->setTitle( $report->getTitle() );
		$this->_objPHPExcel->getProperties()->setSubject( $report->getSubject() );
		$this->_objPHPExcel->getProperties()->setDescription( $report->getDescription() );

		if (count( $this->_data['DivisionData'] ) > 1) {
			$objPHPExcel->getActiveSheet()->setTitle( 'Summary' );
			$this->writeHeader( $this->_queries ['year'] . ' Loaded Vs Sold' );

			$lineNumber = 3;

			$this->writeMonths( 1, $lineNumber );

			$lineNumber += 2;

			$this->writeSummary( $lineNumber );

			$this->writeDivisions();
		} else {
			$this->writeDivisions( false );
		}

		return $this;
	}

	private function writeHeader($text) {
		$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [0] . "1", $text );
		$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [0] . "1" )->applyFromArray( array(
				'font' => array(
						'bold' => true,
						'size' => 14
				),
				'alignment' => array(
						'horizontal' => \PHPExcel_Style_Alignment::HORIZONTAL_LEFT
				)
		) );
	}

	private function writeMonths($i, $lineNumber, $region = false) {
		$headers = array(
				'January',
				'February',
				'March',
				'April',
				'May',
				'June',
				'July',
				'August',
				'September',
				'October',
				'November',
				'December'
		);

		$subHeading = array(
				'Loaded',
				'Sold'
		);

		if ($region) {
			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] .($lineNumber + 1), 'Regional Manager' );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] .($lineNumber + 1) )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'alignment' => array(
							'horizontal' => \PHPExcel_Style_Alignment::HORIZONTAL_CENTER
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'dddddd'
							)
					)
			) );

			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] .($lineNumber + 1), 'Trade Center' );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] .($lineNumber + 1) )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'alignment' => array(
							'horizontal' => \PHPExcel_Style_Alignment::HORIZONTAL_CENTER
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'dddddd'
							)
					)
			) );

			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] .($lineNumber + 1), 'Branch' );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] .($lineNumber + 1) )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'alignment' => array(
							'horizontal' => \PHPExcel_Style_Alignment::HORIZONTAL_CENTER
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'dddddd'
							)
					)
			) );
		}

		foreach ( $headers as $key => $val ) {
			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $val );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i] . $lineNumber )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'alignment' => array(
							'horizontal' => \PHPExcel_Style_Alignment::HORIZONTAL_CENTER
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'aaaaaa'
							)
					)
			) );

			$this->_objPHPExcel->getActiveSheet()->mergeCells( $this->numberToColumn [$i] . $lineNumber . ':' . $this->numberToColumn [$i + 1] . $lineNumber );

			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] .($lineNumber + 1), $subHeading [0] );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i] .($lineNumber + 1) )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'alignment' => array(
							'horizontal' => \PHPExcel_Style_Alignment::HORIZONTAL_CENTER
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'dddddd'
							)
					)
			) );

			$i ++;

			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] .($lineNumber + 1), $subHeading [1] );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i] .($lineNumber + 1) )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'alignment' => array(
							'horizontal' => \PHPExcel_Style_Alignment::HORIZONTAL_CENTER
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'dddddd'
							)
					)
			) );

			$i ++;
		}
	}

	private function writeSummary($lineNumber)
	{
		$summary = array();
		foreach ( $this->_data['DivisionData'] as $division_name => $division ) {
			foreach ( $division as $manager ) {
				foreach ( $manager as $month_name => $month ) {
					if (!isset($summary[$division_name][$month_name]))
					{
						$summary[$division_name][$month_name] = array(
								'Sold' 		=> 0,
								'Loaded' 	=> 0
						);
					}
					$summary[$division_name][$month_name]['Sold'] 	+= $month['Sold'];
					$summary[$division_name][$month_name]['Loaded'] += $month['Sold'];
					$summary[$division_name][$month_name]['Loaded'] += $month['Relist'];
					$summary[$division_name][$month_name]['Loaded'] += $month['Undone'];

					/* if ($month ['jobState'] == 'Sold') {
						$summary [$division_name] [$month ['month']] ['Sold'] =(isset( $summary [$division_name] [$month ['month']] ['Sold'] ) ? $summary [$division_name] [$month ['month']] ['Sold'] + $month ['counter'] : $month ['counter']);
						$summary [$division_name] [$month ['month']] ['Loaded'] =(isset( $summary [$division_name] [$month ['month']] ['Loaded'] ) ? $summary [$division_name] [$month ['month']] ['Loaded'] + $month ['counter'] : $month ['counter']);
					} else {
						$summary [$division_name] [$month ['month']] ['Loaded'] =(isset( $summary [$division_name] [$month ['month']] ['Loaded'] ) ? $summary [$division_name] [$month ['month']] ['Loaded'] + $month ['counter'] : $month ['counter']);
					} */
				}
			}
		}

		$lineNumber = $this->writeData( $summary, $lineNumber );
	}

	private function writeDivisions($summary = true)
	{
		foreach ( $this->_data['CompanyData'] as $division_name => $division ) {
			if ($summary) {
				$this->_objPHPExcel->createSheet();
				$this->_objPHPExcel->setActiveSheetIndex( $this->_objPHPExcel->getSheetCount() - 1 );
			}
			$title = preg_replace( "/[^a-zA-Z0-9\s]+/", "", $division_name );
			$title = substr( $title, 0, 30 );

			if ($title) {
				$this->_objPHPExcel->getActiveSheet()->setTitle( $title );
			} else {
				$this->_objPHPExcel->getActiveSheet()->setTitle( 'Unknown' );
			}

			$this->writeHeader( $division_name . ' ' . $this->_queries ['year'] . ' Loaded Vs Sold' );

			$lineNumber = 3;

			foreach ( $division as $manager_name => $manager ) {
				$this->writeMonths( 0, $lineNumber, true );

				$lineNumber += 2;

				$summary = array();

				foreach ( $manager as $company_name => $company ) {
					foreach ( $company as $month_name => $month ) {
						if (!isset($summary[$company_name][$month_name]))
						{
							$summary[$company_name][$month_name] = array(
									'Sold' 		=> 0,
									'Loaded' 	=> 0
							);
						}
						$summary[$company_name][$month_name]['Sold'] 	+= $month['Sold'];
						$summary[$company_name][$month_name]['Loaded'] 	+= $month['Sold'];
						$summary[$company_name][$month_name]['Loaded']  += $month['Relist'];
						$summary[$company_name][$month_name]['Loaded']  += $month['Undone'];

						/* if ($month ['jobState'] == 'Sold') {
							$summary [$company_name] [$month ['month']] ['Sold'] =(isset( $summary [$month ['companyName']] [$month ['month']] ['Sold'] ) ? $summary [$month ['companyName']] [$month ['month']] ['Sold'] + $month ['counter'] : $month ['counter']);
							$summary [$company_name] [$month ['month']] ['Loaded'] =(isset( $summary [$month ['companyName']] [$month ['month']] ['Loaded'] ) ? $summary [$month ['companyName']] [$month ['month']] ['Loaded'] + $month ['counter'] : $month ['counter']);
						} else {
							$summary [$company_name] [$month ['month']] ['Loaded'] =(isset( $summary [$month ['companyName']] [$month ['month']] ['Loaded'] ) ? $summary [$month ['companyName']] [$month ['month']] ['Loaded'] + $month ['counter'] : $month ['counter']);
						} */
					}
				}

				$lineNumber = $this->writeData( $summary, $lineNumber, $manager_name, $division_name );
			}
		}
	}

	private function writeData($data, $lineNumber, $region = false, $division = false)
	{
		$months = array();

		for($i = 1; $i <= 12; $i ++) {
			$months [] = $this->_queries ['year'] . '-' . str_pad(( string ) $i, 2, '0', STR_PAD_LEFT );
		}

		$first = true;

		$totals = array();
		foreach ( $months as $month ) {
			$totals [$month] ['Loaded'] = 0;
			$totals [$month] ['Sold'] = 0;
		}

		foreach ( $data as $key => $entry ) {
			$i = 0;
			if ($region) {
				if ($first) {
					$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber,($region == ' ' ? 'Other' : $region) );
					$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i] . $lineNumber )->applyFromArray( array(
							'font' => array(
									'bold' => true,
									'size' => 8
							)
					) );
					$first = false;
				}
				$i ++;

				$tradeCenter = isset($this->_data['TradeCenterData']["$division.$key"])
					? $this->_data['TradeCenterData']["$division.$key"]
					: '';
				$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $tradeCenter );
				$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] . $lineNumber )->applyFromArray( array(
						'font' => array(
								'bold' => true,
								'size' => 8
						)
				) );

				$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $key );
				$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] . $lineNumber )->applyFromArray( array(
						'font' => array(
								'bold' => true,
								'size' => 8
						)
				) );

			} else {
				$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $key );
				$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] . $lineNumber )->applyFromArray( array(
						'font' => array(
								'bold' => true,
								'size' => 8
						)
				) );
			}

			foreach ( $months as $month ) {
				$sold =(isset( $entry [$month] ['Sold'] ) ? $entry [$month] ['Sold'] : '0');

				$loaded =(isset( $entry [$month] ['Loaded'] ) ? $entry [$month] ['Loaded'] : '0');

				$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $loaded );
				$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] . $lineNumber )->applyFromArray( array(
						'font' => array(
								'bold' => true,
								'size' => 8
						)
				) );
				$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $sold );
				$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] . $lineNumber )->applyFromArray( array(
						'font' => array(
								'bold' => true,
								'size' => 8
						)
				) );

				$totals [$month] ['Loaded'] += $loaded;
				$totals [$month] ['Sold'] += $sold;
			}
			$lineNumber ++;
		}

		// Totals
		$i = 1;
		if ($region) {
			$i = 3;
		}

		foreach ( $totals as $total ) {

			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $total ['Loaded'] );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] . $lineNumber )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'dddddd'
							)
					)
			) );
			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] . $lineNumber, $total ['Sold'] );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i] . $lineNumber )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					),
					'fill' => array(
							'type' => \PHPExcel_Style_Fill::FILL_SOLID,
							'color' => array(
									'rgb' => 'dddddd'
							)
					)
			) );

			$this->_objPHPExcel->getActiveSheet()->setCellValue( $this->numberToColumn [$i] .($lineNumber + 1),($total ['Loaded'] ? number_format( $total ['Sold'] / $total ['Loaded'] * 100, 2 ) : 0) . '%' );
			$this->_objPHPExcel->getActiveSheet()->getStyle( $this->numberToColumn [$i ++] .($lineNumber + 1) )->applyFromArray( array(
					'font' => array(
							'bold' => true,
							'size' => 8
					)
			) );
		}

		return $lineNumber + 1;
	}
}
