<?php
namespace Stock\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="vehicle_type")
 */
class Type
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = 'Update';
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = 'Update';

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="Model", inversedBy="types")
	 * @ORM\JoinColumn(name="vehicle_model_id", referencedColumnName="id")
	 **/
	protected $model;

	/**
	 * @ORM\ManyToOne(targetEntity="Category")
	 * @ORM\JoinColumn(name="vehicle_category_id", referencedColumnName="id")
	 **/
	protected $category;

	/**
	 * @ORM\Column(type="string", length=150)
	 */
	protected $name;

	/**
	 * @ORM\Column(type="string", length=10, name="mm_code")
	 */
	protected $mmCode;

	/**
	 * @ORM\ManyToOne(targetEntity="Year")
	 * @ORM\JoinColumn(nullable=false, name="intro_vehicle_year_id")
	 **/
	private $introYear;

	/**
	 * @ORM\Column(type="smallint", name="intro_month")
	 */
	protected $introMonth;

	/**
	 * @ORM\ManyToOne(targetEntity="Year")
	 * @ORM\JoinColumn(nullable=true, name="disc_vehicle_year_id")
	 **/
	protected $discYear;

	/**
	 * @ORM\Column(type="smallint", nullable=true, name="disc_month")
	 */
	protected $discMonth;

	/**
	 * @ORM\Column(type="smallint", nullable=true)
	 */
	protected $cylinders;

	/**
	 * @ORM\Column(type="integer", nullable=true, name="cubic_capacity")
	 */
	protected $cubicCapacity;

	/**
	 * @ORM\Column(type="smallint", nullable=true)
	 */
	protected $kilowatts;

	/**
	 * @ORM\Column(type="string", nullable=true, length=25, name="body_type")
	 */
	protected $bodyType;

	/**
	 * @ORM\Column(type="smallint", nullable=true)
	 */
	protected $doors;


	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="smallint", nullable=true, name="create_version", options={"unsigned"=true})
	 */
	protected $createVersion;

	/**
	 * @ORM\Column(type="smallint", nullable=true, name="update_version", options={"unsigned"=true})
	 */
	protected $updateVersion;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;


	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(array $expand = array(), array $intersect = array())
	{
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['model']))
			&& in_array('model', $expand)
			&& $data['model'] = $this->model->toArray($expand, $intersect);
		($includeAll || isset($intersect['category']))
			&& in_array('category', $expand)
			&& $data['category'] = $this->category->toArray($expand, $intersect);
		($includeAll || isset($intersect['name']))
			&& $data['name'] = $this->name;
		($includeAll || isset($intersect['mmCode']))
			&& $data['mmCode'] = $this->mmCode;
		($includeAll || isset($intersect['introYear']))
			&& in_array('introYear', $expand)
			&& $data['introYear'] = $this->introYear->toArray($expand, $intersect);
		($includeAll || isset($intersect['introMonth']))
			&& $data['introMonth'] = $this->introMonth;
		($includeAll || isset($intersect['discYear']))
			&& in_array('discYear', $expand)
			&& $data['discYear'] = !is_null($this->discYear)
				? $this->discYear->toArray($expand, $intersect)
				: null;
		($includeAll || isset($intersect['discMonth']))
			&& $data['discMonth'] = $this->discMonth;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format(\Utility\Registry::getConfigParam('DateTimeFormat'))
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format(\Utility\Registry::getConfigParam('DateTimeFormat'))
				: null;
		return $data;
	}

	/**
	 * Convert the object to an array for synchronization.
	 * @return array
	 */
	public function toSynchArray()
	{
		return array(
				'id'        => $this->id,
				'model'     => $this->model->id,
				'category'  => $this->category->id,
				'name'      => $this->name,
				'mmCode'    => $this->mmCode,
				'introYear' => $this->introYear->id,
				'diskYear'  => !is_null($this->discYear)
									? $this->discYear->id
									: 0
		);
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['model'])
			&& $this->model  = $data['model'];
		isset($data['category'])
			&& $this->category  = $data['category'];
		isset($data['name'])
			&& $this->name  = $data['name'];
		isset($data['mmCode'])
			&& $this->mmCode  = $data['mmCode'];
		isset($data['introYear'])
			&& $this->introYear  = $data['introYear'];
		isset($data['introMonth'])
			&& $this->introMonth  = $data['introMonth'];
		isset($data['discYear'])
			&& $this->discYear  = $data['discYear'];
		isset($data['discMonth'])
			&& $this->discMonth  = $data['discMonth'];
		isset($data['cylinders'])
			&& $this->cylinders  = $data['cylinders'];
		isset($data['cubicCapacity'])
			&& $this->cubicCapacity  = $data['cubicCapacity'];
		isset($data['kilowatts'])
			&& $this->kilowatts  = $data['kilowatts'];
		isset($data['bodyType'])
			&& $this->bodyType  = $data['bodyType'];
		isset($data['doors'])
			&& $this->doors  = $data['doors'];
		isset($data['createVersion'])
			&& $this->createVersion  = $data['createVersion'];
		isset($data['updateVersion'])
			&& $this->updateVersion  = $data['updateVersion'];
	}

}
