<?php
namespace Stock\Pdf;



class Tuhpi extends \Utility\Service\PdfTemplate
{


	/**
	 * PDF title.
	 * @var string
	 */
	protected $_title = 'HPI Report';
	/**
	 * @var \Stock\Entity\Stock
	 */
	protected $jobRecord;
	/**
	 * @var boolean
	 */
	protected $showCustomerDetails = false;
	/**
	 * @var boolean
	 */
	protected $showValuationDetails = false;
	/**
	 * @var boolean
	 */
	protected $showPricingDetails = false;
	/**
	 * @var boolean
	 */
	protected $showVehicleDetails = false;
	/**
	 * @var boolean
	 */
	protected $showVehicleNotes = false;
	/**
	 * @var boolean
	 */
	protected $showDamageDetails = false;
	/**
	 * @var boolean
	 */
	protected $showAccessoryDetails = false;
	/**
	 * @var boolean
	 */
	protected $showSignatureLines = false;
	/**
	 * @var boolean
	 */
	protected $showTermsAndConditions = false;
	/**
	 * @var boolean
	 */
	protected $showOffersReceived = false;
	/**
	 * @var boolean
	 */
	protected $showOfferMade = false;


	/**
	 * Utility for easy parameter retrieval.
	 * @param string $param
	 * @param unknown $default
	 * @return unknown
	 */
	protected function getStockParam($param, $default)
	{
		return !is_null($this->jobRecord->$param)
			? $this->jobRecord->$param
			: $default;
	}

	/**
	 * Utility for easy parameter retrieval.
	 * @param string $param
	 * @param unknown $default
	 * @return unknown
	 */
	protected function getStockSubParam($ref, $param, $default)
	{
		$ref = !is_null($this->jobRecord->$ref)
			? $this->jobRecord->$ref
			: null;
		return is_object($ref)
			? $ref->$param
			: $default;
	}

	/**
	 * Utility for easy parameter retrieval.
	 * @param string $param
	 * @param unknown $default
	 * @return unknown
	 */
	protected function getValuationParam($param, $default)
	{
		return !is_null($this->jobRecord->valuation)
				&& !is_null($this->jobRecord->valuation->$param)
			? $this->jobRecord->valuation->$param
			: $default;
	}


	/**
	 * Build the content.
	 */
	public function build()
	{
		#-> Set data item to work with.
		$authData = \Utility\Registry::getAuthData();
		$this->jobRecord = $this->_input['jobRecord'];

        $recordId = $this->_input['contract']->PdfTemplate['recordId'];

        if(!empty($recordId) && '' != $recordId)
        {
            $result = $this->em->createQuery(
                'SELECT tuhpi '
                . 'FROM \Stock\Entity\Tuhpi tuhpi '
                . 'JOIN tuhpi.stock stock '
                . 'WHERE tuhpi.id = :id '
                . 'ORDER BY tuhpi.created ASC '
            )
                ->setParameter( 'id' , $recordId );
        }
        else
        {
            $result = $this->em->createQuery(
                'SELECT tuhpi '
                . 'FROM \Stock\Entity\Tuhpi tuhpi '
                . 'JOIN tuhpi.stock stock '
                . 'WHERE stock.id = :stockId '
                . ' AND IDENTITY(tuhpi.company) = :companyId '
                . 'ORDER BY tuhpi.created ASC '
            )
                ->setParameter( 'stockId' , $this->jobRecord->id )
                ->setParameter( 'companyId' , $authData['company']['id'] );
        }

//		$result = $this->em->createQuery(
//				'SELECT tuhpi '
//				. 'FROM \Stock\Entity\Tuhpi tuhpi '
//				. 'JOIN tuhpi.stock stock '
//				. 'WHERE stock.id = :stockId '
//				. ' AND IDENTITY(tuhpi.company) = :companyId '
//				. 'ORDER BY tuhpi.created ASC '
//			)
//			->setParameter( 'stockId' , $this->jobRecord->id )
//			->setParameter( 'companyId' , $authData['company']['id'] )
//			->getResult();

        $result = $result->getResult();

		foreach ($result as $data)
		{
			$companyProfileOffice = $data->company->contact->office;
			$companyProfilefax = $data->company->contact->fax;
			$companyRegion = $data->company->region->name;
			$companyCity = $data->company->city->name;
			$companyPostalCode = $data->company->postalCode;
			$companyStreet = $data->company->street;
			$companyDealerType = $data->company->dealerType;
			$companyClientType = $data->company->clientType;
			$companyCompanyRegistrationNumber = $data->company->companyRegistrationNumber;
			$companyBusinessName = $data->company->businessName;


			$f_financeProvider = $data->f_financeProvider;
			$f_financeBranch = $data->f_financeBranch;
			$f_agreementOrAccountNumber = $data->f_agreementOrAccountNumber;
			$f_agreementType = $data->f_agreementType;
			$f_telephoneNumber = $data->f_telephoneNumber;
			$f_resultCode = $data->f_resultCode;


			$fh_data = unserialize($data->fh_data);

			$ah_data = unserialize($data->ah_data);

			$si_policeStation = $data->si_policeStation;
			$si_caseNumber = $data->si_caseNumber;
			$si_dateStolen = $data->si_dateStolen;
			$si_vinMatch = $data->si_vinMatch;
			$si_engineMatch = $data->si_engineMatch;
			$si_registrationMatch = $data->si_registrationMatch;
			$si_resultCode = $data->si_resultCode;

			$ai_company = $data->ai_company;
			$ai_referenceNumber = $data->ai_referenceNumber;
			$ai_alertReason = $data->ai_alertReason;
			$ai_telephoneNumber = $data->ai_telephoneNumber;
			$ai_resultCode = $data->ai_resultCode;

			$ivid_responseCode = $data->ivid_responseCode;
			$ivid_responseDescription = $data->ivid_responseDescription;
			$ivid_referenceNumber = $data->ivid_referenceNumber;
			$ivid_contactNumber = $data->ivid_contactNumber;
			$ivid_resultCode = $data->ivid_resultCode;


			$mi_company = $data->mi_company;
			$mi_referenceNumber = $data->mi_referenceNumber;
			$mi_contactNumber = $data->mi_contactNumber;
			$mi_dateApplied = $data->mi_dateApplied;
			$mi_resultCode = $data->mi_resultCode;


			$vesa_certificateNumber = $data->vesa_CertificateNumber;
			$vesa_dateFitted = $data->vesa_dateFitted;
			$vesa_brandName = $data->vesa_brandName;
			$vesa_brandModel = $data->vesa_brandModel;
			$vesa_deviceDescriptionType = $data->vesa_deviceDescriptionType;
			$vesa_resultCode = $data->vesa_resultCode;


			$rh_data = unserialize($data->rh_data);


			$eh_data = unserialize($data->eh_data);


			$mh_data = unserialize($data->mh_data);


			$vc_month = $data->vc_month;
			$vc_year = $data->vc_year;
			$vc_newOptionCode = $data->vc_newOptionCode;
			$vc_usedOptionCode = $data->vc_usedOptionCode;
			$vc_maximumRange = $data->vc_maximumRange;
			$vc_minimunRange = $data->vc_minimunRange;
			$vc_newPrice = $data->vc_newPrice;
			$vc_retailPrice = $data->vc_retailPrice;
			$vc_tradePrice = $data->vc_tradePrice;
			$vc_resultCode = $data->vc_resultCode;

			$vvi_guideMonth = $data->vvi_guideMonth;
			$vvi_guideYear = $data->vvi_guideYear;
			$vvi_newListPrice = $data->vvi_newListPrice;
			$vvi_retailPrice = $data->vvi_retailPrice;
			$vvi_tradePrice = $data->vvi_tradePrice;
			$vvi_costPrice = $data->vvi_costPrice;
			$vvi_adjustedRetailPrice_mileageAndCondition = $data->vvi_adjustedRetailPrice_mileageAndCondition;
			$vvi_AdjustedTradePrice_MileageAndCondition = $data->vvi_AdjustedTradePrice_MileageAndCondition;
			$vvi_adjustedRetailPrice = $data->vvi_adjustedRetailPrice;
			$vvi_adjustedTradePrice = $data->vvi_adjustedTradePrice;
			$vvi_estimatedRetailPrice = $data->vvi_estimatedRetailPrice;
			$vvi_estimatedTradePrice = $data->vvi_estimatedTradePrice;
			$vvi_estimatedCostPrice = $data->vvi_estimatedCostPrice ;
			$vvi_adjustedEstimatedRetailPrice_mileageAndCondition = $data->vvi_adjustedEstimatedRetailPrice_mileageAndCondition = 0;
			$vvi_adjustedEstimatedTradePrice_mileageAndCondition = $data->vvi_adjustedEstimatedTradePrice_mileageAndCondition = 0;
			$vvi_adjustedEstimatedCostPrice_mileageAndCondition = $data->vvi_adjustedEstimatedCostPrice_mileageAndCondition = 0;
			$vvi_adjustedEstimatedRetailPrice = $data->vvi_adjustedEstimatedRetailPrice;
			$vvi_adjustedEstimatedTradePrice = $data->vvi_adjustedEstimatedTradePrice;
			$vvi_adjustedEstimatedCostPrice = $data->vvi_adjustedEstimatedCostPrice;
			$vvi_resultCode = $data->vvi_resultCode;

			$error_code = $data->error_code;

		}

		$dateTimeFormat = \Utility\Definitions\Locale::getDateTimeFormat();
		$currencyPrefix = \Utility\Definitions\Locale::getCurrencyPrefix();
		$authData = \Utility\Registry::getAuthData();
		$msg = 'NO RECORD FOUND ON OUR DATABASE';

		//\Utility\Debug::errorLog('$authData', $authData['']);

		$test = '* Year of Manufacturer - The year of manufacture and the year of first registration normally coincides. If not, the difference is caused by vehicles being
				manufactured/imported late in one year but being sold and first registered as new in a later year.
				TransUnion HPI is a registered financial services provider (22938).
				_______________________________________________________________________________________________________________________
				Please check this confirmation carefully to ensure that you enquired on the correct VIN/Engine number. Should you have any queries, please contact
				Customer Services at: (0861) 507 507
				Report Date: 2014/08/15  11:22 AM ';

		$dateOfPrint = 'Date of print HPI: <span style="color:#333;">' . date('Y-m-d H:i:s') . '</span>';
		$printedBy   = 'Printed by HPI: <span style="color:#333;">' . $authData['firstName']
					 . ' ' . $authData['familyName'] . '</span>';
		$referenceNo = 'Reference no.: <span style="color:#333;">' . $this->jobRecord->referenceNumber . '</span>';


		$this->_logo = '
				<h1 style="font-size:20px; margin:0 auto; text-align:center">TRANSUNION REPORT</h1>
				<div style="width:100%"></div>

				<div style="display:block; width:115px; height:82px; margin-left:640px; margin-top:-40px; " >
					<img  style="" width="115px" height="82px" src="' . __DIR__ . '/../../../../../public/img/app/logos/tuhpi.jpg" style="padding:0;margin:0;width:100%;" />
				</div>
				';

		/* --------------------------------- USER, TransUnion --------------------------------- */



		$type = $this->jobRecord->type;
		$intro ='
			<table style="width:100%; margin-top:-20px;">
				<tr>Adobe Acrobat 11 Pro
				    <td style="width:180px; font-weight:bold;">FOR ATTENTION : </td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight:bold;"></td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight:bold;">' . $companyBusinessName . '</td>
				    <td style="padding-left:20px; font-weight: normal;"> </td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight:bold;">TransUnion</td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;">User : ' . $authData['firstName']. ' ' . $authData['familyName'] . '</td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;">Reg. No. 94/04014/07</td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;">' . $companyStreet . '</td>
				    <td style="padding-left:20px; font-weight: normal;"> </td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;">P.O. Box 1240 </td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;"></td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;">Randburg</td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;">' . $companyRegion . '</td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;">2125</td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;">' . $companyCity . '</td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;"></td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;">' . $companyPostalCode . '</td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;"> </td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;">Tel: ' . $companyProfileOffice . '</td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;">Tel: (0861) 504 504 </td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;">Fax:' . $companyProfilefax . '</td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;">Fax: (0861) 506 506 </td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: bold;">Client Reference No.: </td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: bold;">Reference No.: ' . $this->jobRecord->referenceNumber . ' </td>

				</tr>
				<tr>
				    <td style="width:180px; font-weight: normal;"></td>
				    <td style="padding-left:20px; font-weight: normal;"></td>
				    <td style="width:150px;"></td>
					<td style="width:220px; font-weight: normal;"><span style="font-weight: normal;"><strong>Date of Enquiry :</strong>    ' . date('Y-m-d H:i:s') . '</span>  </td>

				  </tr>
		</table>
		<br />

		<span style="font-size: 10px; font-weight: normal; width:100%; float:left;"> Thank you for your enquiry. The information you have submitted has been checked against our records, the result of the search is displayed below. </span>
		<br /><br />

		<div style="text-align:center; width:100%; "><h2 style="font-size:16px;">TransUnion Vehicle Enquiry</h2></div>
		';
		$this->append($intro);
		/* $container = $this
		->newTable(
				array(30, 5, 30, 5, 30),
				'margin:0;font-size:10px;vertical-align:top;',
				'',
				'padding:10px 0 0 0;'
		)
		->setCellValue(0, 0, $intro);

		$this->append($container->publish()); */


		/* --------------------------------- INFORMATION SUPPLIED /  INFORMATION VERIFIED --------------------------------- */

		$vehicleInfo ='
			<table style="width:100%">
					<tr style="padding:10px;">
					    <td style="width:380px; font-weight: bold; background: #d3d3d3; border-color:#000; padding:5px; padding-right:0px; border-top:1px solid #333; border-left: 1px solid #333; border-bottom:2px solid #333; border-right:1px solid #333">
							INFORMATION SUPPLIED
						</td>
						<td style="width:330px; font-weight: bold;  background: #d3d3d3;border-color:#000;border-left: 0px solid #000;padding:5px; border-top:1px solid #333; border-right: 1px solid #333; border-bottom:2px solid #333 ">
							INFORMATION VERIFIED
						</td>
					</tr>

			</table>

			<table style="width:100%">
				<tr>
				    <td style="width:150px; font-weight: bold;">Manufacturer : </td>
				    <td style="width:100px; padding-left:20px; font-weight: normal;">' . $type->model->make->name . '</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $data->vci_matchManufacturer . '&nbsp;</td>

				</tr>
				<tr>
				    <td style="width:150px; font-weight: bold;">Model Description: </td>
				    <td style="width:120px; padding-left:20px; font-weight: normal;">' . $type->name . '</td>
				    <td style="width:100px;"></td>
					<td style="width:160px; font-weight: normal;">' . $data->vci_matchModel . '&nbsp;</td>

				</tr>
				<tr>
				    <td style="width:150px; font-weight: bold;">VIN or Chassis No. :</td>
				    <td style="width:100px; padding-left:20px; font-weight: normal;">' . $this->getStockParam('vinNumber', '') . '</td>
				    <td style="width:100px;"></td>
					<td style="width:180px; font-weight: normal;">' . $data->vci_matchVinorChassis . '&nbsp;</td>

				</tr>
				<tr>
				    <td style="width:150px; font-weight: bold;">Engine No. : </td>
				    <td style="width:100px; padding-left:20px; font-weight: normal;">' . $this->getStockParam('engineNumber', '') . '</td>
				    <td style="width:100px;"></td>
					<td style="width:180px; font-weight: normal;">' . $data->vci_matchEngineNumber . '&nbsp;</td>

				</tr>
				<tr>
				    <td style="width:150px; font-weight: bold;">Registration No. :</td>
				    <td style="width:100px; padding-left:20px; font-weight: normal;">' . $this->getStockParam('registrationNumber', '') . '</td>
				    <td style="width:100px;"></td>
					<td style="width:180px; font-weight: normal;">' . $data->vci_matchVehicleRegistration . '&nbsp;</td>

				</tr>
				<tr>
				    <td style="width:150px; font-weight: bold;">Colour :</td>
				    <td style="width:100px; padding-left:20px; font-weight: normal;">' . $this->getStockSubParam('exteriorColour', 'name', '') . '</td>
				    <td style="width:100px;"></td>
					<td style="width:180px; font-weight: normal;">' . $data->vci_matchColour . '&nbsp;</td>

				</tr>
				<tr>
				    <td style="width:150px; font-weight: bold;">Year of Manufacture :</td>
				    <td style="width:100px; padding-left:20px; font-weight: normal;">' . $this->getStockSubParam('vehicleYear', 'name', '') . '</td>
				    <td style="width:100px;"></td>
					<td style="width:180px; font-weight: normal;">' . $data->vci_matchYear . '&nbsp;</td>

				</tr>

		</table>
		<br />

		<div style="text-align:center; margin:0 auto; width:100%; color:#860406 "><h2 style="font-size:16px;">' . $data->vci_matchString . '&nbsp;</h2></div>
		<br />

	    <table style="width:100%">
				<tr style="padding:10px;">
					    <td style="width:730px; font-weight: bold; background: #d3d3d3; border-color:#000; padding:5px; padding-right:0px; border-top:1px solid #333; border-left: 1px solid #333; border-bottom:2px solid #333; border-right:1px solid #333;">
							ADDITIONAL INFORMATION REGISTERED
						</td>
				</tr>
		</table>
		';
		$this->append($vehicleInfo);



		/* --------------------------------- FINANCE INTEREST --------------------------------- */



		$financeIntrest ='
				<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
					<h2 style="font-size:12px; font-weight:bold;">FINANCE INTEREST</h2>
				</div>

			<table style="width:100%">
				<tr>
				    <td style="width:250px; font-weight: bold;">Finance Provider : </td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $f_financeProvider . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Finance Branch : </td>
				   <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $f_financeBranch . '</td>

				</tr>

				<tr>
				    <td style="width:250px; font-weight: bold;">Agreement Or Account Number:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $f_agreementOrAccountNumber . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Agreement Type: </td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $f_agreementType . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Telephone Number:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $f_telephoneNumber . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Result Code:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $f_resultCode . '</td>

				</tr>

			</table>
		<br />
		';


		if ($f_financeProvider == '' || $f_financeBranch == '' || $f_agreementOrAccountNumber == '' || $f_agreementType == '' || $f_telephoneNumber == '')
		{
			$msgHeading = '<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
					<h2 style="font-size:12px; font-weight:bold;">FINANCE INTEREST</h2>
				</div>';
			$this->append($msgHeading);
			$this->append($msg . '<br /><br />');
		}
		else
		{
			$this->append($financeIntrest);
		}
		//$this->_page++;

		/* --------------------------------- FINANCE HISTORY INTEREST --------------------------------- */
		$financeHistoryIntrest = '
					<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
						<h2 style="font-size:12px; font-weight:bold;">FINANCE HISTORY INTEREST</h2>
					</div>

				<table style="width:100%">';
		foreach ($fh_data as $fh)
		{

            $resultCodeFh = isset($fh['resultCode']) ? $fh['resultCode'] : '';

			$financeHistoryIntrest .= '
					<tr>
					    <td style="width:250px; font-weight: bold;">Finance House : </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $fh['financeHouse'] . '</td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Agreement Or Account Number:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $fh['agreementOrAccountNumber'] . '</td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Agreement Type: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $fh['agreementType'] . '</td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Telephone Number:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $fh['telephoneNumber'] . '</td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Start Date : </td>
					   <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $fh['startDate'] . '</td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">End Date : </td>
					   <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $fh['endDate'] . '</td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Result Code:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $resultCodeFh . '</td>

					</tr>
					<tr><td colspan="3"><hr/></td></tr>';
		}
		if (empty($fh_data))
		{
			$financeHistoryIntrest .= '<tr><td colspan="3">' . $msg . '</td></tr>';
		}
		$financeHistoryIntrest .= '
			</table>
			<br />
			';



		$this->append($financeHistoryIntrest);
		//$this->_page++;



		/* --------------------------------- POLICE STOLEN INTEREST --------------------------------- */

		$policeStolen ='

				<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
					<h2 style="font-size:12px; font-weight:bold;">POLICE STOLEN INTEREST</h2>
				</div>

			<table style="width:100%">
				<tr>
					<td style="width:80px; font-weight: bold;">Description</td>
				    <td style="width:80px; font-weight: bold; ">Status</td>
				    <td style="width:80px; font-weight: bold;">Police Station </td>
				    <td style="width:80px; font-weight: bold;">SAP Case Number</td>
					<td style="width:80px; font-weight: bold;">Date Stolen</td>
					<td style="width:80px; font-weight: bold;">Vin Match</td>
				    <td style="width:80px; font-weight: bold;">Engine Match</td>
					<td style="width:80px; font-weight: bold;">Registration Match</td>

				</tr>
				<tr>
				 	<td style="width:80px; font-weight: normal;">Successful Enquiry</td>
				    <td style="width:80px; font-weight: normal;">' . $si_resultCode . '</td>
					<td style="width:80px; font-weight: normal;">' . $si_policeStation . '</td>
					<td style="width:80px; font-weight: normal;">' . $si_caseNumber . '</td>
				    <td style="width:80px; font-weight: normal;">' . $si_dateStolen . '</td>
					<td style="width:80px; font-weight: normal;">' . $si_vinMatch . '</td>
					<td style="width:80px; font-weight: normal;">' . $si_engineMatch . ' </td>
				    <td style="width:80px; font-weight: normal;">' . $si_registrationMatch . '</td>
				</tr>

			</table>
		<br />
		';


		if ($si_resultCode == '' || $si_policeStation == '' || $si_caseNumber == '' || $si_dateStolen == '' || $si_dateStolen == '' || $si_vinMatch == '' || $si_engineMatch == '' || $si_registrationMatch == '' )
		{
			$msgHeading = '<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
						<h2 style="font-size:12px; font-weight:bold;">POLICE STOLEN INTEREST</h2>
						</div>

						<table style="width:100%">';
			$this->append($msgHeading);
			$this->append('<tr><td colspan="3">' . $msg . '</td></tr>');
            $this->append('</table><br />');

		}
		else
		{
			$this->append($policeStolen);
		}
//		$this->_page++;

		/* --------------------------------- ACCIDENT HISTORY --------------------------------- */


        $emptyLine ='
			<table style="width:100%">
				<tr>
					<td style="width:80px; font-weight: bold;">Description</td>
				</tr>
			</table>
		<br />
		';

        $this->append($emptyLine);

        $accidentHistory = '<br />
					<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
						<h2 style="font-size:12px; font-weight:bold;">ACCIDENT HISTORY</h2>
					</div>

				<table style="width:100%">';


		foreach ($ah_data as $ah)
		{
			$accidentHistory .= '
					<tr>
					    <td style="width:250px; font-weight: bold;">Assessment Date: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $ah['assessmentDate'] . '</td>
					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Vehicle Description:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $ah['vehicleDescription'] . '</td>
					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Vin Match: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $ah['vinMatch'] . '</td>
					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Reg Match:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;"> ' . $ah['regMatch'] . ' </td>
					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Estimated Repair Cost: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $ah['estimatedRepairCost'] . '</td>
					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Result Code:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $ah['resultCode'] . '</td>

					</tr>
					<tr><td colspan="3"><hr/></td></tr>
			';
		}
		if (empty($ah_data))
		{
			$financeHistoryIntrest .= '<tr><td colspan="3">' . $msg . '</td></tr>';
		}
		$accidentHistory .= '
			</table>
			<br />';

		$this->append($accidentHistory);
		//$this->_page++;



		/* --------------------------------- MICRODOT --------------------------------- */

		$microdot  ='
				<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
					<h2 style="font-size:12px; font-weight:bold;">MICRODOT</h2>
				</div>

			<table style="width:100%">
				<tr>
				    <td style="width:250px; font-weight: bold;">Company: </td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $mi_company . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Reference Number:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $mi_referenceNumber . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Contact Number: </td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $mi_contactNumber . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Date Applied:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $mi_dateApplied . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Result Code:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $mi_resultCode . '</td>

				</tr>

			</table>
		<br />
		';
		if ($mi_company == '' || $mi_referenceNumber == '' || $mi_contactNumber == '' || $mi_dateApplied = '')
		{

			$msgHeading = '<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
			<h2 style="font-size:12px; font-weight:bold;">MICRODOT</h2>
			</div>';
			$this->append($msgHeading);
			$this->append($msg . '<br /><br />');
		}
		else
		{
			$this->append($microdot);
		}


		/* --------------------------------- VESA SECURITY DEVICE HISTORY --------------------------------- */

		$vesaInformation ='&nbsp;<br/>
				<div style="text-align:left; margin:0 0 0 290; width:100%; color:#000; text-decoration:underline; ">
					<h2 style="font-size:12px; font-weight:bold;">VESA SECURITY DEVICE HISTORY</h2>
				</div>

			<table style="width:100%">
				<tr>
				    <td style="width:250px; font-weight: bold;">Certificate Number: </td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $vesa_certificateNumber . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Date Fitted:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $vesa_dateFitted . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Brand Name: </td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $vesa_brandName . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Brand Model:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $vesa_brandModel . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Device Description Type: </td>
				   <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $vesa_deviceDescriptionType . '</td>

				</tr>
				<tr>
				    <td style="width:250px; font-weight: bold;">Result Code:</td>
				    <td style="width:100px;"></td>
					<td style="width:200px; font-weight: normal;">' . $vesa_resultCode . '</td>

				</tr>

			</table>
		<br />
		';

		if ($vesa_certificateNumber == '' || $vesa_dateFitted == '' || $vesa_brandName == '' || $vesa_brandModel = '' || $vesa_deviceDescriptionType == '')
		{
			$msgHeading = '<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
								<h2 style="font-size:12px; font-weight:bold;">VESA SECURITY DEVICE HISTORY</h2>
							</div>';
			$this->append($msgHeading);
			$this->append($msg . '<br /><br />');
		}
		else
		{
			$this->append($vesaInformation);
		}
		//$this->_page++;


		/* --------------------------------- MILEAGES --------------------------------- */



		$mileage = '
			<div style="text-align:center; margin:0 auto; width:100%; color:#000; ">
			<h2 style="font-size:12px; font-weight:bold;">&nbsp; </h2>
			</div>
			<div style="text-align:left; margin:0 0 0 350; width:100%; color:#000; text-decoration:underline; ">
			<h2 style="font-size:12px; font-weight:bold;">MILEAGES</h2>
			</div>

			<table style="width:100%">';
		foreach ($mh_data as $mh)
		{

            $resultCodeMh = isset($fh['resultCode']) ? $fh['resultCode'] : '';

			$mileage .= '
					<tr>
					    <td style="width:250px; font-weight: bold;">Source: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $mh['source'] . '</td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Vehicle Mileage:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $mh['vehicleMileage'] . ' </td>

					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Date Registered: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $mh['dateRegistered'] . '</td>

					</tr>

					<tr>
					    <td style="width:250px; font-weight: bold;">Result Code:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $resultCodeMh . '</td>

					</tr>
					<tr><td colspan="3"><hr/></td></tr>';
		}
		if (empty($mh_data))
		{
			$financeHistoryIntrest .= '<tr><td colspan="3">' . $msg . '</td></tr>';
		}
		$mileage .= '
			</table>
			<br />';

		$this->append($mileage);


		/* --------------------------------- PREVIOUS INQUIRIES --------------------------------- */

		$enquiryHistory = '
			<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
			<h2 style="font-size:12px; font-weight:bold;">PREVIOUS INQUIRIES</h2>
			</div>

			<table style="width:100%">';
		foreach ($eh_data as $eh)
		{
			$enquiryHistory .= '
					<tr>
					    <td style="width:250px; font-weight: bold;">Customer type: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $eh['source'] . '</td>
					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Transaction Date:</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;"> ' . $eh['transactionDate'] . '</td>
					</tr>
					<tr><td colspan="3"><hr/></td></tr>
			';
		}
		if (empty($eh_data))
		{
			$financeHistoryIntrest .= '<tr><td colspan="3">' . $msg . '</td></tr>';
		}
		$enquiryHistory .= '
			</table>
			<br />';


		$this->append($enquiryHistory);

		/* --------------------------------- PREVIOUS REGISTRATIONS --------------------------------- */

		$registrationHistory = '
			<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
			<h2 style="font-size:12px; font-weight:bold;">PREVIOUS REGISTRATIONS</h2>
			</div>
				<table style="width:100%">';
		foreach ($rh_data as $rh)
		{
			$registrationHistory .= '
					<tr>
					    <td style="width:250px; font-weight: bold;">Registration Number: </td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $rh['registrationNumber'] . '</td>
					</tr>
					<tr>
					    <td style="width:250px; font-weight: bold;">Date Registered</td>
					    <td style="width:100px;"></td>
						<td style="width:200px; font-weight: normal;">' . $rh['dateRegistered'] . '</td>
					</tr>
					<tr><td colspan="3"><hr/></td></tr>
			';
		}
		if (empty($rh_data))
		{
			$financeHistoryIntrest .= '<tr><td colspan="3">' . $msg . '</td></tr>';
		}
		$registrationHistory .= '
			</table>
			<br />';


		$this->append($registrationHistory);
		//$this->_page++;

		/* --------------------------------- IVID RESPONSE x --------------------------------- */

		$vesaInformation ='
				<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
					<h2 style="font-size:12px; font-weight:bold;">IVID RESPONSE</h2>
				</div>

			<table style="width:100%">
				<tr>
				    <td style="width:60px; font-weight: bold;">Code </td>
				    <td style="width:340px; font-weight: bold;">Description</td>
					<td style="width:150px; font-weight: bold;">IVID Reference Number</td>
					<td style="width:150px; font-weight: bold;">IVID Contact Number</td>
				</tr>
				<tr>
				    <td style="width:60px; font-weight: normal;">' . $ivid_responseCode . '</td>
					<td style="width:340px; font-weight: normal;">' . $ivid_responseDescription . '</td>
					<td style="width:150px; font-weight: normal;">' . $ivid_referenceNumber . '</td>
				    <td style="width:150px; font-weight: normal;">' . $ivid_contactNumber . '</td>
				</tr>
			</table>
		<br />
		';

		if ($ivid_responseCode == '' || $ivid_responseDescription == '' || $ivid_referenceNumber == '' || $ivid_contactNumber == '')
		{
			$msgHeading = '<div style="text-align:center; margin:0 auto; width:100%; color:#000; text-decoration:underline; ">
							<h2 style="font-size:12px; font-weight:bold;">IVID RESPONSE</h2>
						  </div>';
			$this->append($msgHeading);
			$this->append($msg . '<br /><br />');
		}
		else
		{
			$this->append($vesaInformation);
		}


		/* --------------------------------- SIGNATURES --------------------------------- */
		#-> Signatures.
		/* if ($this->showSignatureLines)
		{
			$this->append(
					$this->newSignatureLine()
						->addSignatureLine('Customer (name)', 'Signature', 'Date')
						->addSignatureLine('Manager (name)', 'Signature', 'Date')
						->publish()
					);
		}

		$this->append(
				'<div style="position:absolute;bottom:0;left:0;padding:0;margin:0;width:100%;">'
				. '<table cellpadding="0px" cellspacing="0px" style="border:0px;width:100%;">'
				. '<tr>'
				. '<td style="width:30%;border-bottom: solid 1px #333;font-size:10px;font-weight:bold;color:#000;"><i>'
				. $dateOfPrint . '</i></td>'
				. '<td style="width:5%">&nbsp;</td>'
				. '<td style="width:30%;border-bottom: solid 1px #333;font-size:10px;font-weight:bold;color:#000;"><i>'
				. $printedBy . '</i></td>'
				. '<td style="width:5%">&nbsp;</td>'
				. '<td style="width:30%;border-bottom: solid 1px #333;font-size:10px;font-weight:bold;color:#000;"><i>'
				. $referenceNo . '</i></td>'
				. '</tr>'
				. '</table>'
				. '</div>'
				);
		*/

		/* $this->append(
					'<div style="position:absolute;bottom:0;left:0;padding:0;margin:0;width:100%;">'
					. '<table cellpadding="0px" cellspacing="0px" style="border:0px;width:100%;">'
					. '<tr>'
					. '<td style="width:30%;border-bottom: solid 1px #333;font-size:10px;font-weight:bold;color:#000;"><i>'
					. $dateOfPrint . '</i></td>'
					. '<td style="width:5%">&nbsp;</td>'
					. '<td style="width:30%;border-bottom: solid 1px #333;font-size:10px;font-weight:bold;color:#000;"><i>'
					. $printedBy . '</i></td>'
					. '<td style="width:5%">&nbsp;</td>'
					. '<td style="width:30%;border-bottom: solid 1px #333;font-size:10px;font-weight:bold;color:#000;"><i>'
					. $referenceNo . '</i></td>'
					. '</tr>'
					. '</table>'
					. '</div>'
			); */







		return;
	}


}