<?php

namespace Valuation\DataBin;

/**
 * Manage Valuation Appointments data.
 * @author andre.fourie
 */
class ValuationAppointments extends \Workspace\Service\DataBin
{

  /**
   * @var array
   */
  protected $meta = array(
    'Base' => 'ValuationAppointments',
    'DatasetName' => 'valuationAppointments',
    'Entity' => '\Valuation\Entity\ValuationAppointments',
    'References' => array(
      'timeSlot' => '\Valuation\Entity\ValuationTimeSlots',
      'valuator' => '\Valuation\Entity\ValuationValuators',
      'valuation' => '\Valuation\Entity\Valuation',
    ),
    'Dependants' => array()
  );

  /**
   * @var array
   */
  protected $metaList = array(
    'Contract' => 'Recurring',
    'RequireAuth' => true,
    'RequiredInput' => array(),
    'OptionalInput' => array(
      'Filter' => array(
        'created' => 'Date',
        'updated' => 'Date',
        'appointmentDate' => 'Date',
        'timeSlot' => 'Id',
        'valuator' => 'Id',
        'valuation' => 'Id',
          'appointmentDate' => 'Date',
      )
    ),
    'Filter' => array(),
    'Fields' => array(
      'appointmentDate',
      'timeSlot',
      'valuator',
      'valuation'
    ),
    'Expand' => array()
  );

  /**
   * @var array
   */
  protected $metaCreate = array(
    'Type' => 'Create',
    'Contract' => 'Recurring',
    'Flags' => array(),
      'Expand'   => array(
          'timeSlot',
          'valuator',
          'valuation',
//          'category',
//          'make',
//          'model',
//          'type',
//          'stock'
      ),
    'RelatedEntityFromInput' => array(
        'TimeSlot' => array(
            'Workflow' => '\Valuation\Workflow',
            'Service' => '\Valuation\Service\ValuationTimeSlots',
            'Field' => 'timeSlot'
        ),
        'Valuator' => array(
            'Workflow' => '\Valuation\Workflow',
            'Service' => '\Valuation\Service\ValuationValuators',
            'Field' => 'valuator'
        ),
        'Valuation' => array(
            'Workflow' => '\Valuation\Workflow',
            'Service' => '\Valuation\Service\Valuation',
            'Field' => 'valuation'
          )
    ),
    'RequiredInput' => array(
      'ValuationAppointments' => array(
        'appointmentDate' => 'DateTime'
      ),
    ),
    'OptionalInput' => array(
        'ValuationAppointments' => array(
            'appointmentCancelled' => 'Boolean',
            'appointmentCancelledComments' => 'String500',
            'appointmentCompleted' => 'Boolean',
            'timeSlot' => 'Id'
        ),
        'Valuation' => array(
            'email'                     => 'String250',
            'firstName'                 => 'String50',
            'familyName'                => 'String50',
            'mobile'                    => 'Mobile',
            'customerAddressStreet'     => 'String150',
            'customerAddressStreetName' => 'String150',
            'customerAddressSuburb'     => 'String150',
            'customerAddressCity'       => 'String150',
            'customerAddressPostalCode' => 'String150',
        ),
    )
  );

  /**
   * @var array
   */
  protected $metaUpdate = array(
    'Type' => 'Update',
    'Contract' => 'UseOnce',
      'RequireAuth' => true,
      'ExecuteAfter' => array(
          'updateSendCustomerEmail'
      ),
      'Expand'   => array(
          'timeSlot',
          'valuator',
          'valuation',
          'stock'
      ),
      'RelatedEntityFromInput' => array(
          'Valuation' => array(
              'Workflow' => '\Valuation\Workflow',
              'Service' => '\Valuation\Service\Valuation',
              'Field' => 'valuation'
          )
      ),
      'RequiredInput' => array(
          'ValuationAppointments' => array(
              'appointmentDate' => 'DateTime'
          )
      ),
      'OptionalInput' => array(
          'ValuationAppointments' => array(
              'appointmentCancelled' => 'Boolean',
              'appointmentCancelledComments' => 'String500',
              'appointmentCompleted' => 'Boolean',
              'timeSlot' => 'Id'
          ),
          'Valuation' => array(
              'email'                     => 'String250',
              'firstName'                 => 'String50',
              'familyName'                => 'String50',
              'mobile'                    => 'Mobile',
              'customerAddressStreet'     => 'String150',
              'customerAddressStreetName' => 'String150',
              'customerAddressSuburb'     => 'String150',
              'customerAddressCity'       => 'String150',
              'customerAddressPostalCode' => 'String150',
          ),
      )
  );


  /**
   * @var array
   */
  protected $metaDelete = array(
    'Contract' => 'UseOnce',
    'RequireAuth' => true
  );



  /**
   * Dynamic filtering.
   */
  public function __construct()
  {

  }

}
