<?php
namespace Valuation\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A valuation.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="customer_public_valuation")
 */
class CustomerPublicValuation
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = 'Build';
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = 'Update';
	/**
	 * Post insert action must be called after new entity is flushed to database.
	 */
	const HAVE_POST_INSERT = true;
	/**
	 * Handle as a job queue for mobile devices.
	 */
	const JOB_QUEUE = true;




	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="customer_hash");
	 */
	protected $customerHash;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="\Stock\Entity\Stock", cascade={"all"})
	 * @ORM\JoinColumn(nullable=false, name="stock_id")
	 **/
	protected $stock;

    /**
     * @ORM\ManyToOne(targetEntity="\Valuation\Entity\Valuation", cascade={"all"})
     * @ORM\JoinColumn(nullable=false, name="valuation_id")
     **/
    protected $valuation;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;

    /**
     * Magic getter to expose protected properties.
     *
     * @param string $property
     * @return mixed
     */
    public function __get($property)
    {
        return $this->$property;
    }

//    /**
//     * Magic setter to save protected properties.
//     *
//     * @param string $property
//     * @param mixed $value
//     */
//    public function __set($property, $value)
//    {
//        if ('password' == $property)
//        {
//            $this->customerHash = md5($value);
//            return;
//        }
//        $this->$property = $value;
//    }

    /**
     * @ORM\PrePersist
     */
    public function setCreateTime()
    {
        $this->created = new \DateTime("now");
    }

    /**
     * @ORM\PreUpdate
     */
    public function setUpdateTime()
    {
        $this->updated = new \DateTime("now");
    }

    /**
     * Convert the object to an array.
     * @param array $expand
     * @param array $intersect
     * @param boolean $showIdentifiers
     * @param integer $expandAll
     * @return array
     */
    public function toArray(
        array $expand = array(), array $intersect = array(),
        $showIdentifiers = false, $expandAll = 0
    )
    {
        $intersect = array_flip($intersect);
        $dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
        $includeAll = empty($intersect);
        $data = array();
        ($includeAll || isset($intersect['id']))
        && $data['id'] = $this->id;
        ($includeAll || isset($intersect['customerHash']))
        && $data['customerHash'] = $this->customerHash;
        ($includeAll || isset($intersect['stock']))
        && $data['stock'] = $this->stock;
        ($includeAll || isset($intersect['valuation']))
        && $data['valuation'] = $this->valuation;
        ($includeAll || isset($intersect['created']))
        && $data['created'] = !is_null($this->created)
            ? $this->created->format($dateTimeFormat)
            : null;
        ($includeAll || isset($intersect['updated']))
        && $data['updated'] = !is_null($this->updated)
            ? $this->updated->format($dateTimeFormat)
            : null;
        ($includeAll || isset($intersect['archived']))
        && $data['archived'] = $this->archived;
        return $data;
    }

    /**
     * Populate from an array.
     * @param array $data
     */
    public function fromArray($data = array())
    {
        isset($data['id'])
        && $this->id = $data['id'];
        isset($data['customerHash'])
        && $this->customerHash = $data['customerHash'];
        isset($data['stock'])
        && $this->stock = $data['stock'];
        isset($data['valuation'])
        && $this->valuation = $data['valuation'];

    }

}

