<?php
namespace Valuation\Entity;

use Doctrine\ORM\Mapping as ORM;
use Zend\XmlRpc\Value\DateTime;


/**
 * A valuation.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="valuation_appointments")
 */
class ValuationAppointments
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = 'Build';
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = 'Update';
	/**
	 * Post insert action must be called after new entity is flushed to database.
	 */
	const HAVE_POST_INSERT = true;
	/**
	 * Handle as a job queue for mobile devices.
	 */
	const JOB_QUEUE = true;

	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

    /**
     * @ORM\ManyToOne(targetEntity="\Valuation\Entity\ValuationTimeSlots", cascade={"all"})
     * @ORM\JoinColumn(nullable=false, name="time_slot_id")
     **/
    protected $timeSlot;

    /**
     * @ORM\ManyToOne(targetEntity="\User\Entity\Profile", cascade={"all"})
     * @ORM\JoinColumn(nullable=false, name="valuator_profile_id")
     **/
    protected $valuator;

    /**
     * @ORM\ManyToOne(targetEntity="\Valuation\Entity\Valuation", cascade={"all"})
     * @ORM\JoinColumn(nullable=false, name="valuation_id")
     **/
    protected $valuation;


    /**
     * @ORM\Column(type="datetime", nullable=false, name="appointment_date");
     */
    protected $appointmentDate;

    /**
     * @ORM\Column(type="boolean", name="appointment_completed");
     */
    protected $appointmentCompleted = false;


    /**
     * @ORM\Column(type="boolean", name="appointment_cancelled");
     */
    protected $appointmentCancelled = false;
    /**
     * @ORM\Column(type="datetime", name="appointment_date_cancelled", nullable=true);
     */
    protected $appointmentDateCancelled;
    /**
     * @ORM\Column(type="text", name="appointment_cancelled_comments", nullable=true);
     */
    protected $appointmentCancelledComments;




	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;


    /**
     * Magic getter to expose protected properties.
     *
     * @param string $property
     * @return mixed
     */
    public function __get($property)
    {
        return $this->$property;
    }

    /**
     * Magic setter to save protected properties.
     *
     * @param string $property
     * @param mixed $value
     */
    public function __set($property, $value)
    {
        $this->$property = $value;
    }

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");

        if($this->appointmentCancelled)
        {
            $this->appointmentDateCancelled = new \DateTime("now");
        }
	}



	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;

        ($includeAll || isset($intersect['timeSlot']))
        && $data['timeSlot'] = (in_array('timeSlot', $expand) || $expandAll || $showIdentifiers)
        && !is_null($this->timeSlot)
            ? (!$showIdentifiers || in_array('timeSlot', $expand) ? $this->timeSlot->toArray(
                $expand, $intersect, $showIdentifiers, ($expandAll - 1)
            ) : $this->timeSlot->id)
            : null;


        ($includeAll || isset($intersect['valuator']))
        && $data['valuator'] = (in_array('valuator', $expand) || $expandAll || $showIdentifiers)
        && !is_null($this->valuator)
            ? (!$showIdentifiers || in_array('valuator', $expand) ? $this->valuator->toArray(
                $expand, $intersect, $showIdentifiers, ($expandAll - 1)
            ) : $this->valuator->id)
            : null;

        ($includeAll || isset($intersect['valuation']))
        && $data['valuation'] = (in_array('valuation', $expand) || $expandAll || $showIdentifiers)
        && !is_null($this->valuation)
            ? (!$showIdentifiers || in_array('valuation', $expand) ? $this->valuation->toArray(
                $expand, $intersect, $showIdentifiers, ($expandAll - 1)
            ) : $this->valuation->id)
            : null;


        ($includeAll || isset($intersect['appointmentDate']))
        && $data['appointmentDate'] = !is_null($this->appointmentDate)
            ? $this->appointmentDate->format($dateTimeFormat)
            : null;

		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
        ($includeAll || isset($intersect['archived']))
            && $data['archived'] = $this->archived;

        ($includeAll || isset($intersect['appointmentCompleted']))
        && $data['appointmentCompleted'] = $this->appointmentCompleted;
        ($includeAll || isset($intersect['appointmentCancelled']))
        && $data['appointmentCancelled'] = $this->appointmentCancelled;

        ($includeAll || isset($intersect['appointmentDateCancelled']))
        && $data['appointmentDateCancelled'] = !is_null($this->appointmentDateCancelled)
            ? $this->appointmentDateCancelled->format($dateTimeFormat)
            : null;

        ($includeAll || isset($intersect['appointmentCancelledComments']))
        && $data['appointmentCancelledComments'] = $this->appointmentCancelledComments;

		return $data;
	}



	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
//        unset($data['id']);
//        unset($data['timeSlot']);
//        unset($data['valuator']);
//        unset($data['valuation']);
//        unset($data['appointmentCompleted']);
//        unset($data['appointmentCancelled']);
//        unset($data['appointmentDateCancelled']);
//        unset($data['appointmentDateCancelled']);
//        unset($data['appointmentCancelledComments']);


//        \Utility\Debug::errorLog('$data',$data);

		isset($data['id'])
			&& $this->id = $data['id'];

		isset($data['timeSlot'])
			&& $this->timeSlot = $data['timeSlot'];
		isset($data['valuator'])
			&& $this->valuator = $data['valuator'];
        isset($data['valuation'])
            && $this->valuation = $data['valuation'];
        isset($data['appointmentDate'])
        && $this->appointmentDate = (is_object($data['appointmentDate'])
            ? $data['appointmentDate']
            : new \DateTime($data['appointmentDate']));
        isset($data['appointmentCompleted'])
        && $this->appointmentCompleted = $data['appointmentCompleted'];
        isset($data['appointmentCancelled'])
        && $this->appointmentCancelled = $data['appointmentCancelled'];

        isset($data['appointmentDateCancelled'])
        && $this->appointmentDateCancelled = (is_object($data['appointmentDateCancelled'])
            ? $data['appointmentDateCancelled']
            : new \DateTime($data['appointmentDateCancelled']));

        isset($data['appointmentCancelledComments'])
        && $this->appointmentCancelledComments = $data['appointmentCancelledComments'];


//        if (isset($data['appointmentDate']))
//        {
//            $data['appointmentDate'] = explode(' ', $data['appointmentDate']);
//            $this->appointmentDate = new \DateTime($data['appointmentDate'][0]);
//        }
//
//
//        \Utility\Debug::errorLog('$data[appointmentDate]',$data['appointmentDate']);
//
//        \Utility\Debug::errorLog('$this->appointmentDate',$this->appointmentDate);
//
//        $this->updated = new \DateTime("now");
//
//        $this->appointmentDateCancelled = new \DateTime("now");

	}

    /**
     * Convert the object to an array for synchronization.
     * @return array
     */
    public function toSynchArray()
    {
        return array(
            'id'   			                => $this->id,
            'appointmentDate' 	            => $this->appointmentDate,
            'appointmentCompleted' 	        => $this->appointmentCompleted,
            'appointmentCancelled' 	        => $this->appointmentCancelled,
            'appointmentDateCancelled' 	    => $this->appointmentDateCancelled,
            'appointmentCancelledComments' 	=> $this->appointmentCancelledComments
        );
    }


}

