<?php
namespace Valuation\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * An xml-rpc client.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="xmlrpc_auth")
 */
class XmlRpc
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=50, nullable=false, name="client_name");
	 */
	protected $clientName;

	/**
	 * @ORM\Column(type="string", length=42, nullable=false, name="client_api_id");
	 */
	protected $clientApiId;

	/**
	 * @ORM\Column(type="string", length=42, nullable=false, name="password_hash");
	 */
	protected $passwordHash;

	/**
	 * @ORM\Column(type="string", length=42, nullable=true, name="auth_token");
	 */
	protected $authToken;

	/**
	 * @ORM\Column(type="string", length=40, nullable=true, name="ip_address");
	 */
	protected $ipAddress;

	/**
	 * @ORM\Column(type="string", length=250, nullable=false, name="callback_url");
	 */
	protected $callbackUrl;

	/**
	 * @ORM\Column(type="boolean", name="trigger_sent_to_sales");
	 */
	protected $triggerSentToSales = true;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		if ('password' == $property)
		{
			$this->passwordHash = md5($value);
			return;
		}
		if ('ipAddress' == $property)
		{
			$this->authToken = md5($this->clientApiId . time() . $this->passwordHash);
			$this->ipAddress = $value;
			return;
		}
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
		$this->clientApiId = md5(time() . 'xmlrpc-client');
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['clientName']))
			&& $data['clientName'] = $this->clientName;
		($includeAll || isset($intersect['clientApiId']))
			&& $data['clientApiId'] = $this->clientApiId;
		($includeAll || isset($intersect['ipAddress']))
			&& $data['ipAddress'] = $this->ipAddress;
		($includeAll || isset($intersect['callbackUrl']))
			&& $data['callbackUrl'] = $this->callbackUrl;
		($includeAll || isset($intersect['triggerSentToSales']))
			&& $data['triggerSentToSales'] = $this->triggerSentToSales;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['archived']))
			&& $data['archived'] = $this->archived;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['clientName'])
			&& $this->clientName = $data['clientName'];
		isset($data['password'])
			&& $this->passwordHash = md5($data['password']);
		isset($data['callbackUrl'])
			&& $this->callbackUrl = $data['callbackUrl'];
		isset($data['triggerSentToSales'])
			&& $this->triggerSentToSales = $data['triggerSentToSales'];
		isset($data['ipAddress'])
			&& $this->ipAddress = $data['ipAddress'];
		isset($data['ipAddress'])
			&& $this->authToken = md5($this->clientApiId . time() . $this->passwordHash);
	}



}

