<?php
namespace Valuation\Report;



use Zend\Console\Console;
/**
 * Stock Contyrol Report.
 * @author andre.fourie
 */
class Valuations extends \Utility\Service\Report
{



	/**
	 * @var string
	 */
	protected $_title  = 'Valuations Report';
	/**
	 * @var string
	 */
	protected $_subject = 'Valuations';
	/**
	 * @var string
	 */
	protected $_description = 'Historical report for valuations.';
	/**
	 * @var array
	 */
	protected $_notes = array(
			'Confidential information, generated using Bid 4 Cars, for more information visit bid4cars.com.na'
			);
	/**
	 * @var array
	 */
	protected $_headers = array(
			'Registration Number',
			'Make',
			'Model',
			'Derivative',
			'MM Code',
			'Year',
			'KM\'s',
			'Trade Price',
			'Retail Price',
			'Valuation Date',
			'Loaded on Price Guide',
			'Date First Offer Received',
			'Highest Offer Received',
			'Lowest Offer Received',
			'Total Offers Received',
			'Customer Name',
			'Customer Surname',
			'Valuator Name',
			'Valuator Surname',
			'Sales Person Name',
			'Sales Person Surname'
			);
	/**
	 * @var array
	 */
	protected $_fields = array(
			'stock_registrationNumber',
			'make_name',
			'model_name',
			'type_name',
			'type_mmCode',
			'year_name',
			'stock_km',
			'stock_tradePrice',
			'stock_retailPrice',
            'stock_listPrice',
			'valuation_created',
			'priceGuide_created',
			'firstOfferDate',
			'highestOffer',
			'lowestOffer',
			'numOffers',
			'valuation_firstName',
			'valuation_familyName',
			'valuator_firstName',
			'valuator_familyName',
			'sales_firstName',
			'sales_familyName'
			);
	/**
	 * @var array
	 */
	protected $_totals = array(
			'stock_tradePrice',
			'stock_retailPrice',
            'stock_listPrice',
			'highestOffer',
			'lowestOffer'
			);
	/**
	 * @var array
	 */
	protected $_currencyFields = array(
			'stock_tradePrice',
			'stock_retailPrice',
            'stock_listPrice',
			'highestOffer',
			'lowestOffer'
			);



	/**
	 * Build the dataset.
	 */
	public function build()
	{
		#-> Prepare parameters.
		// hier 
		$selection = array(
				'stock', 'type', 'damageTotal', 'model', 'make', 'valuation', 'sales', 'valuator', 'priceGuide',
				'MAX(offer.amount) AS highestOffer',
				'MIN(offer.amount) AS lowestOffer',
				'COUNT(offer.id) AS numOffers',
				'MIN(offer.created) AS firstOfferDate'
		);
		$join      = array();
		$leftJion  = array();
		$where     = array();
		$params    = array();
		$query = 'SELECT [SELECTION] '
				. 'FROM \Valuation\Entity\Valuation valuation '
				. ' JOIN valuation.stock stock '
				. ' JOIN stock.type type '
				. ' JOIN stock.damageTotal damageTotal '
				. ' JOIN type.model model '
				. ' JOIN model.make make '
				. ' [JOIN] '
				. ' LEFT JOIN valuation.valuatedBy valuator '
				. ' LEFT JOIN valuation.salesProfile sales '
				. ' LEFT JOIN stock.priceGuide priceGuide '
				. ' LEFT JOIN priceGuide.offers offer '
				. ' [LEFT-JOIN] '
				. '[WHERE] '
				. 'GROUP BY valuation.id '
				. 'ORDER BY valuation.created ASC';

		#-> Construct details.
		$authData = \Utility\Registry::getAuthData();
		switch (\Utility\Registry::getUserType())
		{
			case 'B4C User':
				$groupFilter = \Utility\Registry::getSudo('Group', false);
				if ($groupFilter)
				{
					$this->_queries['Group'] = \Utility\Registry::getSudo('GroupName');
					$join[]  = 'JOIN stock.company company';
					$where[] = 'IDENTITY(company.group) = :group';
					$params['group'] = $groupFilter;
				}
				$divisionFilter = \Utility\Registry::getSudo('Division', false);
				if ($divisionFilter)
				{
					$this->_queries['Division'] = \Utility\Registry::getSudo('DivisionName');
					$join[]  = 'JOIN stock.company company';
					$where[] = 'IDENTITY(company.groupDivision) = :groupDivision';
					$params['groupDivision'] = $divisionFilter;
				}
				$companyFilter = \Utility\Registry::getSudo('Company', false);
				if ($companyFilter)
				{
					$this->_queries['Company'] = \Utility\Registry::getSudo('CompanyName');
					$where[] = 'IDENTITY(stock.company) = :company';
					$params['company'] = $companyFilter;
				}
				break;
			case 'Group User':
				$join[]  = 'JOIN stock.company company';
				$where[] = 'IDENTITY(company.group) = :group';
				$params['group'] = $authData['company']['group']['id'];
				$divisionFilter = \Utility\Registry::getSudo('Division', false);
				if ($divisionFilter)
				{
					$this->_queries['Division'] = \Utility\Registry::getSudo('DivisionName');
					$join[]  = 'JOIN stock.company company';
					$where[] = 'IDENTITY(company.groupDivision) = :groupDivision';
					$params['groupDivision'] = $divisionFilter;
				}
				$companyFilter = \Utility\Registry::getSudo('Company', false);
				if ($companyFilter)
				{
					$this->_queries['Company'] = \Utility\Registry::getSudo('CompanyName');
					$where[] = 'IDENTITY(stock.company) = :company';
					$params['company'] = $companyFilter;
				}
				break;
			case 'Dealer Principle':
				$join[]  = 'JOIN stock.company company';
				$where[] = 'IDENTITY(company.groupDivision) = :groupDivision';
				$params['groupDivision'] = $authData['company']['groupDivision']['id'];
				$companyFilter = \Utility\Registry::getSudo('Company', false);
				if ($companyFilter)
				{
					$this->_queries['Company'] = \Utility\Registry::getSudo('CompanyName');
					$where[] = 'IDENTITY(stock.company) = :company';
					$params['company'] = $companyFilter;
				}
				break;
			default:
				$where[] = 'IDENTITY(stock.company) = :company';
				$params['company'] = $authData['company']['id'];
				break;
		}
		if (isset($this->_input['jobState']))
		{
			$this->_queries['Status'] = $this->_input['jobState'];
			$where[] = 'valuation.jobState >= :jobState';
			$params['jobState'] = $this->_input['jobState'];
		}
		if (isset($this->_input['dateFrom']))
		{
			$this->_queries['Date Range From'] = $this->_input['dateFrom'];
			$where[] = 'valuation.created >= :dateFrom';
			$params['dateFrom'] = new \DateTime($this->_input['dateFrom']);
		}
		if (isset($this->_input['dateTo']))
		{
			$this->_queries['Date Range Until'] = $this->_input['dateTo'];
			$where[] = 'valuation.created <= :dateTo';
			$params['dateTo'] = new \DateTime($this->_input['dateTo']);
		}

		#-> Finalize query.
		$query = str_replace(array(
				'[SELECTION]', '[JOIN]', '[LEFT-JOIN]', '[WHERE]'
		), array(
				implode(', ', $selection) . ' ',
				implode(' ', $join) . ' ',
				implode(' ', $leftJion) . ' ',
				!empty($where)
					? 'WHERE ' . implode(' AND ', $where) . ' '
					: ' '
		), $query);

		#-> Collect data.
		$query = $this->em->createQuery($query);
		!empty($params)
			&& $query->setParameters($params);
		$this->_data = $query->getScalarResult();
		$data = $this->_data = $query->getScalarResult();
		//console.log($this->_data = $query->getScalarResult());
	
		var_dump('test',$data);
	}

}
