<?php
namespace Workspace\UseCase;



/**
 * Base condition set container.
 * @author andre.fourie
 */
abstract class ConditionBase
{

	/**
	 * Usefull constants
	 */
	const BASE_OR  = 'or';
	const BASE_AND = 'and';

	/**
	 * @var string
	 */
	protected $base;
	/**
	 * @var array
	 */
	protected $conditionSet = array();



	/**
	 * Retrieve condition set.
	 * @return array
	 */
	public function getPrerequisites()
	{
		return array(
				'Operator' => $this->base,
				'Conditions' => $this->conditionSet
		);
	}

	/**
	 * Add sub condition(s).
	 * @param \Workspace\UseCase\ConditionBase $condition
	 * @return \Workspace\UseCase\ConditionBase
	 */
	public function addSubGroup(\Workspace\UseCase\ConditionBase $condition)
	{
		$this->conditionSet[] = $condition->getPrerequisites();
		return $this;
	}

	/**
	 * Equals.
	 * @param string $input
	 * @param unknown $value
	 * @return \Workspace\UseCase\ConditionBase
	 */
	public function equal($input, $value)
	{
		return $this->addCondition('=', $input, $value);
	}

	/**
	 * Not equals.
	 * @param string $input
	 * @param unknown $value
	 * @return \Workspace\UseCase\ConditionBase
	 */
	public function notEqual($input, $value)
	{
		return $this->addCondition('!=', $input, $value);
	}

	/**
	 * Smaller than.
	 * @param string $input
	 * @param unknown $value
	 * @return \Workspace\UseCase\ConditionBase
	 */
	public function smallerThan($input, $value)
	{
		return $this->addCondition('<', $input, $value);
	}

	/**
	 * Greater than.
	 * @param string $input
	 * @param unknown $value
	 * @return \Workspace\UseCase\ConditionBase
	 */
	public function greaterThan($input, $value)
	{
		return $this->addCondition('>', $input, $value);
	}

	/**
	 * Is one of the specified values.
	 * @param string $input
	 * @param array $value
	 * @return \Workspace\UseCase\ConditionBase
	 */
	public function oneOf($input, array $value)
	{
		return $this->addCondition('IN', $input, $value);
	}

	/**
	 * Not one of the specified values.
	 * @param string $input
	 * @param array $value
	 * @return \Workspace\UseCase\ConditionBase
	 */
	public function notOneOf($input, array $value)
	{
		return $this->addCondition('NOT IN', $input, $value);
	}

	/**
	 * Add new condition.
	 * @param string $op
	 * @param string $input
	 * @param unknown $value
	 * @return \Workspace\UseCase\ConditionBase
	 */
	protected function addCondition($op, $input, $value)
	{
		$this->conditionSet[] = array(
				'Input' => $input,
				'Operator' => $op,
				'Value' => $value
		);
		return $this;
	}

}